% function simulateCase5(maxCounts,filename)
% Calculates drilling fluid flowrates and pressures
% Refer to description of Case 1
% (c) IRIS Petroleum
% 15. june 2007 GNY


clear all

filename = 'test1'; 

%Constants
  
g = 9.81;
d = 9220;
h_r = 2150;
h_y = 1650;
rho_w = 1000;
riserID = 0.473;
dsOD = 0.127;
A = pi*riserID^2 - pi*dsOD^2;
p0 = 1e5;


reg = 1000; %Time between set point alteration
maxCounts = 17*reg; % Simulation time


%Initial values
xp(:,1) = [2050;1880;3.409e7];
xpI(:,1) = xp(:,1);
xp2(:,1) = xp(:,1);
x_(:,1) = xp(:,1);
x_2(:,1) = xp(:,1);

%Error covariance matrix
P(:,:,1) = eye(3);

%Jacobian matrices
F = zeros(3);
L = eye(3);
M = eye(2);

%Noise
sigmaw1 = 0.1;
sigmaw2 = 0.1;
sigmaw3 = 0.1;
sigmav1 = 2000;
sigmav2 = 6000;

%Tunable noise matrices
Q = zeros(3);
Q(1,1) = 10000*sigmaw1^2;
Q(2,2) = 10000*sigmaw2^2;
Q(3,3) = 10000*sigmaw3^2;
R = zeros(2);
R(1,1) = 10000*sigmav1^2;
R(2,2) = 10000*sigmav2^2;

%First timestep

u1Array(1) = 1000/60000;
u7Array(1) = 1000/60000;

z_(:,1) = [rho_w * g * (h_r - x_(1,1)) + x_(2,1) * g * x_(1,1) + p0; rho_w * g * (h_r - x_(1,1)) + x_(2,1) * g * x_(1,1) + p0 + x_(2,1) * g * (d - h_r) + x_(3,1) * u1Array(1)];
zp(:,1) = [rho_w * g * (h_r - xp(1,1)) + xp(2,1) * g * xp(1,1) + p0; rho_w * g * (h_r - xp(1,1)) + xp(2,1) * g * xp(1,1) + p0 + xp(2,1) * g * (d - h_r) + xp(3,1) * u1Array(1)];
zpK(:,1) = zp(:,1);
meas(:,1) = [z_(1,1); z_(2,1)];


plotThick = 1;

CASE = 1;
% 1 = AGR
% 2 = ORS

  % Brukes midlertidig til Main Pump
  firstStartUp = 100;
  firstStartUpTime = 1*60;
  firstFlow    =1000/(60*1000);
  startFirstCounter     = 0;

  % Brukes midlertidig til Choke Pump
  secondStartUp = 400;
  secondStartUpTime = 1*60;
  secondFlow    = 0/(60*1000);
  startSecondCounter     = 0;
  
  thirdStartUp = 1000;
  thirdStartUpTime = 1*60;
  thirdFlow    = 0/(60*1000);
  startThirdCounter  = 0;
  
    % Brukes midlertidig til Extraction Pump
  fourthStartUp = 1400;
  fourthStartUpTime = 1*60;
  fourthFlow    = 1000/(60*1000);
  startFourthCounter     = 0;
  
  % Brukes midlertidig til Insertion Pump
  fifthStartUp = 200;
  fifthStartUpTime = 1*60;
  fifthFlow    = 00/(60*1000);
  startFifthCounter     = 0;
  
  u7_old = fourthFlow;%1400/(60*1000);



disp('Initializing...');
timeStep = 1; % 1 second per timeStep
loadlibrary ('WeModForMatlab','WeModForMatlab.h');
caseNumber = 7; 
result = calllib('WeModForMatlab', 'initWeMod',caseNumber);
if result == 1
    disp ('Unable to find previous time log');
    return;
elseif (result == 2 || result ==3 || result ==4)
    disp ('Inconsistency in time log');
    return;    
elseif (result ==5)
    disp ('License has expired!');
    return;  
elseif result ~= 0
    disp ('Error initialization WeMod, is software key installed?');
    return;
end
    
% Defining initial u vector
% u1 - main pump rate, [m3/s] 
% u2 - choke opening, [m2]
% u3 - choke pump annulus pump rate, [m3/s] 
% u4 - fluid density, [kg/m3](alternative: [SG] (relative to water density 1)
% u5 - drillstring velocity  [m/s]
% u6 - choke pump fluid density  [kg/m3]
% u7 - annulus extraction pump flow rate  [m3/s]
% u8 - annulus injection pump flow rate  [m3/s]  
% u9 - annulus injection pump density  [kg/m3]
% u10 - Drillstring Rotation. [RPM]

u1 = 0;
u2 = 1; % m2, more than full opening...
u3 = 0;
u4 = 1804;%1722.4;%1601.6;
u5 = 0;
u6 = 1000;%1722.4;
u7 = 0;
u8 = 0;
u9 = 1804;%1701.6;
u10 = 0;




% DEFINE REFERENCE VALUES HERE:
% reference pressures, annulus velocities etc...
%
% end of reference definitions

% initialize controller
    
% end of initialize controller

% initialize observer

%end of initialize observer

%Define sensor selection
%numSensors = 12; % each having 9 values (pres + 7*flow + temp)
%numBranches = 5; % each having 8 values (pres + 7*flow)
%Define sensor selection
numSensors = 0; % each having 9 values (pres + 7*flow + temp)
numBranches = 4; % each having 8 values (pres + 7*flow)

% Extract sensor names
numSensorNames = calllib('WeModForMatlab','getSensorNameListSize');
% numSensorNames
for SensorNum = 1:numSensorNames
    SensorName = calllib('WeModForMatlab', 'getSensorName',SensorNum-1);
    disp(SensorName)
    save('namefile','SensorName');
end


%loop
disp(sprintf('Simulating to %d [s]',maxCounts));

u1Array = zeros(maxCounts,1);
u2Array = zeros(maxCounts,1);
u3Array = zeros(maxCounts,1);
u4Array = zeros(maxCounts,1);
u5Array = zeros(maxCounts,1);
u6Array = zeros(maxCounts,1);
u7Array = zeros(maxCounts,1);
u8Array = zeros(maxCounts,1);
u9Array = zeros(maxCounts,1);
u10Array = zeros(maxCounts,1);

U = zeros(10,maxCounts);
  spp = zeros(maxCounts,1);
  bhp= zeros(maxCounts,1);
  cdp= zeros(maxCounts,1);
 
  wm_bit= zeros(maxCounts,1);
  wm_pump= zeros(maxCounts,1);
  wm_exit= zeros(maxCounts,1);
  wm_inject= zeros(maxCounts,1);
  wb_bit= zeros(maxCounts,1);
  wb_pump= zeros(maxCounts,1);
  wb_exit= zeros(maxCounts,1);
  wb_inject= zeros(maxCounts,1);
  africFactors = zeros(maxCounts,1);
  dfricFactors = zeros(maxCounts,1);
  
  

  
  
  
  
for timeCount = 1:maxCounts
  %%%%%%%%%%%%%%
  % CONTROLLER %
  %%%%%%%%%%%%%%
  
  u1 = 1000/60000; %Main pump flow

  
  %Tuneable parameters
   Kp = 0.000007;
   Ti = 0.20;
      
      eBHPPresent = zp(2,timeCount-1)/1e5 -bhpRef(timeCount);
      eBHPLast = zp(2,timeCount -2)/1e5 - bhpRef(timeCount);
  
  if((timeCount/timeStep) >= (3) && timeCount/timeStep <= reg)
      
      bhpRef(timeCount) =  1680;%1420; 
    
  end
  if((timeCount/timeStep) >= (reg) && timeCount/timeStep <= 2*reg)
      
      bhpRef(timeCount) =  1675;%1420; 
    
  end
  if((timeCount/timeStep) >= 2*reg && timeCount/timeStep <= 3*reg)
      
      bhpRef(timeCount) =  1670;%1420; 
     
  end
  if((timeCount/timeStep) >= (3*reg) && timeCount/timeStep <= 4*reg)
      
      bhpRef(timeCount) =  1665;%1420; 
     
  end
  if((timeCount/timeStep) >= (4*reg) && timeCount/timeStep <= 5*reg)
      
      bhpRef(timeCount) =  1660;%1420; 
    
  end
  if((timeCount/timeStep) >= (5*reg) && timeCount/timeStep <= 6*reg)
      
      bhpRef(timeCount) =  1655;%1420; 
     
  end
  if((timeCount/timeStep) >= (6*reg) && timeCount/timeStep <= 7*reg)
      
      bhpRef(timeCount) =  1650;%1420; 
   
  end
  if((timeCount/timeStep) >= (7*reg) && timeCount/timeStep <= 8*reg)
      
      bhpRef(timeCount) =  1645;%1420; 
     
  end
  if((timeCount/timeStep) >= (8*reg) && timeCount/timeStep <= 9*reg)
      
      bhpRef(timeCount) =  1640;%1420; 
      
  end
  if((timeCount/timeStep) >= (9*reg) && timeCount/timeStep <= 10*reg)
      
      bhpRef(timeCount) =  1635;%1420; 
     
  end
  if((timeCount/timeStep) >= (10*reg) && timeCount/timeStep <= 11*reg)
      
      bhpRef(timeCount) =  1640;%1420; 
      
  end
  if((timeCount/timeStep) >= (11*reg) && timeCount/timeStep <= 12*reg)
      
      bhpRef(timeCount) =  1645;%1420; 
  
  end
  if((timeCount/timeStep) >= (12*reg) && timeCount/timeStep <= 13*reg)
      
      bhpRef(timeCount) =  1650;%1420; 
     
  end
  if((timeCount/timeStep) >= (13*reg) && timeCount/timeStep <= 14*reg)
      
      bhpRef(timeCount) =  1655;%1420; 
   
  end
  if((timeCount/timeStep) >= (14*reg) && timeCount/timeStep <= 15*reg)
      
      bhpRef(timeCount) =  1660;%1420; 
     
  end
  if((timeCount/timeStep) >= (15*reg) && timeCount/timeStep <= 16*reg)
      
      bhpRef(timeCount) =  1665;%1420; 
     
  end
  if((timeCount/timeStep) >= (16*reg) && timeCount/timeStep <= 17*reg)
      
      bhpRef(timeCount) =  1670;%1420; 
     
  end

   u7 = u7_old + Kp * (1+1/(2*Ti))*eBHPPresent - Kp * (1-1/(2*Ti))*eBHPLast; % PI-controller

  
   
   % Setting maximun pump rate
   
  if(u7 < 0 )
          u7 = 0;
  elseif(u7 > (2000/(60*1000)))
          u7 = 2000/(60*1000);
  end



  
  % Store controlled variables
  u1Array(timeCount) = u1;
  u2Array(timeCount) = u2;
  u3Array(timeCount) = u3;
  u4Array(timeCount) = u4;
  u5Array(timeCount) = u5;
  u6Array(timeCount) = u6;
  u7Array(timeCount) = u7;
  u8Array(timeCount) = u8;
  u9Array(timeCount) = u9;
  u10Array(timeCount) = u10;
 
  
  u = [u1,u2,u3,u4, u5, u6, u7, u8, u9, 10];
  U(:,timeCount) = u; 
  


  calllib('WeModForMatlab', 'setMainPumpVolumeRate',u1);
  calllib('WeModForMatlab', 'setChokeValveOpening',u2);
  calllib('WeModForMatlab', 'setChokePumpVolumeRate',u3);
  calllib('WeModForMatlab', 'setMainPumpDensity',u4);
  calllib('WeModForMatlab', 'setDrillstringVelocity',u5);
  calllib('WeModForMatlab', 'setChokePumpDensity',u6);
  calllib('WeModForMatlab', 'setAnnulusExtractionPumpVolumeRate',u7);
  calllib('WeModForMatlab', 'setAnnulusInjectionPumpVolumeRate',u8);
  calllib('WeModForMatlab', 'setAnnulusInjectionPumpDensity',u9);
  calllib('WeModForMatlab', 'setRotationRate',u10);
  

  maxVals = calllib('WeModForMatlab', 'step');
  y = zeros(maxVals,1);
  % Reading sensor values using get methods
   for yPos = 1:maxVals
    y(yPos)  = calllib('WeModForMatlab', 'getMeasurementUsingPos',yPos-1);% C array indexes start at 0
   end
  

   
  %Friction parameters
  africFactors(timeCount) = calllib('WeModForMatlab', 'getAnnulusFrictionFactorParameter');
  dfricFactors(timeCount)  = calllib('WeModForMatlab', 'getDrillStringFrictionFactorParameter');
   

   
  %Replace with point data structure
  spp(timeCount) = calllib('WeModForMatlab', 'getSPP');  %Stand pipe pressure
  bitp(timeCount) = calllib('WeModForMatlab', 'getPressureAfterBit');   %Bottom hole pressure in annulus
  cdp(timeCount) = calllib('WeModForMatlab', 'getChokePressure');   %Choke differential pressure
  
  bhp(timeCount) = y(26);
  
  BHP = bhp(timeCount)/1e5;
     

        
%    LLString(timeCount) = calllib('WeModForMatlab', 'getLiquidLevelString')
%    LLAnnulus(timeCount) = calllib('WeModForMatlab', 'getLiquidLevelAnnulus');
        
  
%     rTop(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',59);
%     pTop(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',49);
  
   riserBottom(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',54);

 %mud
  wm_bit(timeCount) =  -y(36); %Mass flow rate at bit
  wm_pump(timeCount) = y(46); % at pump
  wm_exit(timeCount) = -y(31); %and at exit
  wm_inject(timeCount) = -y(61); %and at injection pipe

%baryte
  wb_bit(timeCount) = -y(37); %Mass flow rate at bit
  wb_pump(timeCount) = y(47); % at pump
  wb_exit(timeCount) = -y(32); %and at exit
  wb_inject(timeCount) = -y(62); %and at injection pipe

%   for sensorPos = 1:numSensors
%     Y(timeCount,sensorPos,:) = y(1+(8*numBranches)+9*(sensorPos-1)+[1 2 8 9]);
%   end

  
  % ADD OBSERVER CODE HERE
  %
 
  

     freq = 30; %Update rate for the bhp
  steps = 1; %Number of times the filter is run each timestep
  
  
  
  %%%%%%%%%%%%%%%%%%%%
  %Kalman filter
  %%%%%%%%%%%%%%%%%%%%
  if(timeCount>1)
      
     
      if(timeCount<=freq)    
       
          for k=1:steps
              
              if(k==1)
                
                  PK = F * P(:,:,timeCount-1) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-1) + (1/A * (u1Array(timeCount) - u7Array(timeCount)))/steps; xp(2,timeCount-1); xp(3,timeCount-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
                   
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];

                  
              elseif(k>1)
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount) - u7Array(timeCount)))/steps; xpK(2,k-1); xpK(3,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
                  
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];

              end
          end
                
                
            P(:,:,timeCount) = PK;
            x_(:,timeCount) = x_K(:,steps);

            H(:,:,timeCount) = HK;
            K(:,:,timeCount) = KK;

            z_(:,timeCount) = z_K(:,steps);
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xp(:,steps);
            P(:,:,timeCount) = PK;

            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) * u1Array(timeCount)];

        
      else
        
        if(floor((timeCount-1)/freq) == (timeCount-1)/freq && timeCount>=freq)
            bhpUpdate(timeCount) = bhp(timeCount-freq);
            
            for k=1:steps
                
                if(k==1)
                
                  PK = F * P(:,:,timeCount-freq) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-freq) + (1/A * (u1Array(timeCount-freq) - u7Array(timeCount-freq)))/steps; xp(2,timeCount-freq); xp(3,timeCount-freq)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount-freq)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount-freq)];
                   
                  measK(:,k) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];
                  
                elseif(k>1)
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount-freq) - u7Array(timeCount-freq)))/steps; xpK(2,k-1); xpK(3,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount-freq)];
                  
                  measK(:,k) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];
                end

                PI(:,:,timeCount-freq) = PK;
                x_I(:,timeCount-freq) = x_K(:,steps);
                HI(:,:,timeCount-freq) = HK;
                KI(:,:,timeCount-freq) = KK;

                z_I(:,timeCount-freq) = z_K(:,steps);

                measI(:,timeCount-freq) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                xpI(:,timeCount-freq) = xpK(:,steps);
                PI(:,:,timeCount-freq) = PK;
            end


                for j=(timeCount-freq+1):(timeCount)
                PI(:,:,j) = F * PI(:,:,j-1) * F' + L * Q * L';
                x_I(:,j) = [xpI(1,j-1) + 1/A * (u1Array(j) - u7Array(j)); xpI(2,j-1); xpI(3,j-1)];

                HI(:,:,j) = [(x_I(2,j)-rho_w) * g g * x_I(1,j) 0; (x_I(2,j)-rho_w) * g g * x_I(1,j) + g * (d - h_r) u1Array(j)];
                KI(:,:,j) = PI(:,:,j) * HI(:,:,j)' * inv(HI(:,:,j) * PI(:,:,j) * HI(:,:,j)' + M * R + M');

                z_I(:,j) = [rho_w * g * (h_r - x_I(1,j)) + x_I(2,j) * g * x_I(1,j) + p0; rho_w * g * (h_r - x_I(1,j)) + x_I(2,j) * g * x_I(1,j) + p0 + x_I(2,j) * g * (d - h_r) + x_I(3,j) * u1Array(j)];

                measI(:,j) = [riserBottom(j);z_I(2,j)];

                xpI(:,j) = x_I(:,j) + KI(:,:,j) * (measI(:,j) - z_I(:,j));
                zpI(:,j) = [rho_w * g * (h_r - xpI(1,j)) + xpI(2,j) * g * xpI(1,j) + p0; rho_w * g * (h_r - xpI(1,j)) + xpI(2,j) * g * xpI(1,j) + p0 + xpI(2,j) * g * (d - h_r) + xpI(3,j) * u1Array(j)];

                PI(:,:,j) = (eye(3) - KI(:,:,j) * HI(:,:,j)) * PI(:,:,j);
            end

            P(:,:,timeCount) = PI(:,:,timeCount);
            x_(:,timeCount) = x_I(:,timeCount);

            H(:,:,timeCount) = HI(:,:,timeCount);
            K(:,:,timeCount) = KI(:,:,timeCount);

            z_(:,timeCount) = z_I(:,timeCount);
            
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xpI(:,timeCount);
            
            zp(:,timeCount) = zpI(:,timeCount);
            


        else
            
            P(:,:,timeCount) = F * P(:,:,timeCount-1) * F' + L * Q * L';
            x_(:,timeCount) = [xp(1,timeCount-1) + 1/A * (u1Array(timeCount) - u7Array(timeCount)); xp(2,timeCount-1); xp(3,timeCount-1)];

            H(:,:,timeCount) = [(x_(2,timeCount)-rho_w) * g g * x_(1,timeCount) 0; (x_(2,timeCount)-rho_w) * g g * x_(1,timeCount) + g * (d - h_r) u1Array(timeCount)];
            K(:,:,timeCount) = P(:,:,timeCount) * H(:,:,timeCount)' * inv(H(:,:,timeCount) * P(:,:,timeCount) * H(:,:,timeCount)' + M * R + M');

            z_(:,timeCount) = [rho_w * g * (h_r - x_(1,timeCount)) + x_(2,timeCount) * g * x_(1,timeCount) + p0; rho_w * g * (h_r - x_(1,timeCount)) + x_(2,timeCount) * g * x_(1,timeCount) + p0 + x_(2,timeCount) * g * (d - h_r) + x_(3,timeCount) * u1Array(timeCount)];
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = x_(:,timeCount) + K(:,:,timeCount) * (meas(:,timeCount) - z_(:,timeCount));
            P(:,:,timeCount) = (eye(3) - K(:,:,timeCount) * H(:,:,timeCount)) * P(:,:,timeCount);
            
            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) * u1Array(timeCount)];

        end
        
      end
      
      
  end
  % End of observer code
 

  % ADD Evaluation of reference here
  %
  % End of evaluating reference
  
  %Show progress
  if rem(timeCount,floor(100/timeStep)) == 0
    disp(sprintf('Time: %d [s] and BHP: %10.5f [bar]',floor(timeCount*timeStep),bhp(timeCount)/1e5));
  end
end
calllib('WeModForMatlab', 'terminateWeMod',caseNumber); 

disp('Simulation finished... unloading WeModForMatlab.dll..');
%whos



%save(filename,'U','spp','bhp','cdp','Y');
save(filename,'U','spp','bhp','cdp');
unloadlibrary('WeModForMatlab');
disp('Unloaded WeModForMatlab.dll, and plotting...');



% close all;
%PVT = dlmread('oil1.pvt');



%%

%Plots

figure
subplot(2,1,1)
plot(zp(2,:)/1e5-bhp'/1e5)
xlabel('Time [s]')
ylabel('Pressure [bar]')
axis([0 maxCounts -2.5 2.5])
grid on
title('The error between the calculated and the measured bottom hole pressure in the well')
subplot(2,1,2)
plot(zp(1,:)/1e5-riserBottom/1e5)
grid on
axis([0 maxCounts -0.05 0.05])
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('The error between the calculated and the measured pressure at the extraction pump')

figure
subplot(2,1,1)
plot(zp(2,:)/1e5)
hold on
plot(bhp/1e5,'--r')
xlabel('Time [s]')
ylabel('Pressure [bar]')
axis([0 maxCounts 1650 1690])
legend('Calculated','Measured')
title('The bottom hole pressure in the well')
grid on
subplot(2,1,2)
grid on
plot(zp(1,:)/1e5)
hold on
plot(riserBottom/1e5,'--r')
axis([0 maxCounts 350 386])
grid on
legend('Calculated','Measured')
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('The pressure at the extraction pump')

figure
plot(u1Array*60000)
hold on
plot(u7Array*60000,'r')
plot((u1Array-u7Array)*60000,'g')
legend('q_{in}','q_{out}','q_{in}-q_{out}')
xlabel('Time [s]')
ylabel('Flow [l/min]')
title('The flow in the system')
grid on

figure
subplot(2,1,1)
plot(xp(1,:))
xlabel('Time [s]')
ylabel('Height [m]')
title('Height of mud in the riser')
axis([0 maxCounts 1400 2070])
grid on
subplot(2,1,2)
plot(xp(2,:))
xlabel('Time [s]')
ylabel('Weight [kg/m^3]')
title('Density of the mud')
axis([0 maxCounts 1860 1880])
grid on


figure;
plot(u1Array*60000)
hold on
plot(u7Array*60000,'r')
plot(u3Array*60000,'g')
legend('In','Out','Vann')



figure
subplot(3,1,1)
plot(bhp/1e5)
hold on
plot(bhpRef,'--r')
grid on
axis([0 maxCounts 1630 1685])
legend('BHP','Reference')
title('The controlled bottom hole pressure plotted together with the reference pressure')
subplot(3,1,2)
plot(bhp/1e5-bhpRef')
grid on
axis([0 maxCounts -5 5])
title('The difference between the setpoint and the bottom hole pressure')
subplot(3,1,3)
plot(u1Array*60000)
hold on
grid on
plot(u7Array*60000,'r')
axis([0 maxCounts 0 2000])
legend('q_{in}','q_{out}')
title('The flow in the system')




