% function simulateCase5(maxCounts,filename)
% Calculates drilling fluid flowrates and pressures
% Refer to description of Case 1
% (c) IRIS Petroleum
% 15. june 2007 GNY

filename = 'test1'; 

%Constants
  
g = 9.81;
d = 9220;
h_r = 2150;
rho_w = 1000;
riserID = 0.473;
dsOD = 0.127;
A = pi*riserID^2 - pi*dsOD^2;
p0 = 1e5;

%Simulation time

maxCounts = 3000;

%Initial values, friction modelled as a0 + a1 * u1 + a2 * u1^2


xp(:,1) = [2100;1750;10e5;1.5e4;1.5e4];
xpI(:,1) = xp(:,1);
xp2(:,1) = xp(:,1);
x_(:,1) = xp(:,1);
x_2(:,1) = xp(:,1);


P(:,:,1) = eye(5); %Covariance error matrix

%Jacobian matrices

F = zeros(5);
L = eye(5);
M = eye(2);

%Process and measurement noise

sigmaw1 = 0.1;
sigmaw2 = 0.1;
sigmaw3 = 0.1;
sigmav1 = 200;
sigmav2 = 600;

%Process noise covariance matrix

Q = zeros(5);
Q(1,1) = 10000*sigmaw1^2;
Q(2,2) = 10000*sigmaw2^2;
Q(3,3) = 10000*sigmaw2^2;
Q(4,4) = 10000*sigmaw2^2;
Q(5,5) = 10000*sigmaw2^2;

%Measurement noise covariance matrix

R = zeros(2);
R(1,1) = 10000*sigmav1^2;
R(2,2) = 10000*sigmav2^2;

%First timestep

u1Array(1) = 1000/60000;
u7Array(1) = 1000/60000;

z_(:,1) = [rho_w * g * (h_r - x_(1,1)) + x_(2,1) * g * x_(1,1) + p0; rho_w * g * (h_r - x_(1,1)) + x_(2,1) * g * x_(1,1) + p0 + x_(2,1) * g * (d - h_r) + x_(3,1) + x_(4,1) * u1Array(1) + x_(5,1) * u1Array(1)^2];
zp(:,1) = [rho_w * g * (h_r - xp(1,1)) + xp(2,1) * g * xp(1,1) + p0; rho_w * g * (h_r - xp(1,1)) + xp(2,1) * g * xp(1,1) + p0 + xp(2,1) * g * (d - h_r) + xp(3,1) + xp(4,1) * u1Array(1) + xp(5,1) * u1Array(1)^2];

meas(:,1) = [zp(1,1); zp(2,1)];

CASE = 1;
% 1 = AGR
% 2 = ORS

  % Brukes midlertidig til Main Pump
  firstStartUp = 0;
  firstStartUpTime = 1*60;
  firstFlow    =1000/(60*1000);
  startFirstCounter     = 0;

  % Brukes midlertidig til Choke Pump
  secondStartUp = 400;
  secondStartUpTime = 1*60;
  secondFlow    = 0/(60*1000);
  startSecondCounter     = 0;
  
  thirdStartUp = 1000;
  thirdStartUpTime = 1*60;
  thirdFlow    = 0/(60*1000);
  startThirdCounter  = 0;
  
    % Brukes midlertidig til Extraction Pump
  fourthStartUp = 0;
  fourthStartUpTime = 1*60;
  fourthFlow    = 1000/(60*1000);
  startFourthCounter     = 0;
  
  % Brukes midlertidig til Insertion Pump
  fifthStartUp = 200;
  fifthStartUpTime = 1*60;
  fifthFlow    = 00/(60*1000);
  startFifthCounter     = 0;
  
  u7_old = fourthFlow;%1400/(60*1000);



disp('Initializing...');
timeStep = 1; % 1 second per timeStep
loadlibrary ('WeModForMatlab','WeModForMatlab.h');
caseNumber = 7; 
result = calllib('WeModForMatlab', 'initWeMod',caseNumber);
if result == 1
    disp ('Unable to find previous time log');
    return;
elseif (result == 2 || result ==3 || result ==4)
    disp ('Inconsistency in time log');
    return;    
elseif (result ==5)
    disp ('License has expired!');
    return;  
elseif result ~= 0
    disp ('Error initialization WeMod, is software key installed?');
    return;
end
    
% Defining initial u vector
% u1 - main pump rate, [m3/s] 
% u2 - choke opening, [m2]
% u3 - choke pump annulus pump rate, [m3/s] 
% u4 - fluid density, [kg/m3](alternative: [SG] (relative to water density 1)
% u5 - drillstring velocity  [m/s]
% u6 - choke pump fluid density  [kg/m3]
% u7 - annulus extraction pump flow rate  [m3/s]
% u8 - annulus injection pump flow rate  [m3/s]  
% u9 - annulus injection pump density  [kg/m3]
% u10 - Drillstring Rotation. [RPM]

u1 = 0;
u2 = 1; % m2, more than full opening...
u3 = 0;
u4 = 1804;%1722.4;%1601.6;
u5 = 0;
u6 = 1000;%1722.4;
u7 = 0;
u8 = 0;
u9 = 1804;%1701.6;
u10 = 0;




% DEFINE REFERENCE VALUES HERE:
% reference pressures, annulus velocities etc...
%
% end of reference definitions

% initialize controller
    
% end of initialize controller

% initialize observer

%end of initialize observer

%Define sensor selection
%numSensors = 12; % each having 9 values (pres + 7*flow + temp)
%numBranches = 5; % each having 8 values (pres + 7*flow)
%Define sensor selection
numSensors = 0; % each having 9 values (pres + 7*flow + temp)
numBranches = 4; % each having 8 values (pres + 7*flow)

% Extract sensor names
numSensorNames = calllib('WeModForMatlab','getSensorNameListSize');
% numSensorNames
for SensorNum = 1:numSensorNames
    SensorName = calllib('WeModForMatlab', 'getSensorName',SensorNum-1);
    disp(SensorName)
    save('namefile','SensorName');
end


%loop
disp(sprintf('Simulating to %d [s]',maxCounts));

u1Array = zeros(maxCounts,1);
u2Array = zeros(maxCounts,1);
u3Array = zeros(maxCounts,1);
u4Array = zeros(maxCounts,1);
u5Array = zeros(maxCounts,1);
u6Array = zeros(maxCounts,1);
u7Array = zeros(maxCounts,1);
u8Array = zeros(maxCounts,1);
u9Array = zeros(maxCounts,1);
u10Array = zeros(maxCounts,1);

U = zeros(10,maxCounts);
  spp = zeros(maxCounts,1);
  bhp= zeros(maxCounts,1);
  cdp= zeros(maxCounts,1);
 
  wm_bit= zeros(maxCounts,1);
  wm_pump= zeros(maxCounts,1);
  wm_exit= zeros(maxCounts,1);
  wm_inject= zeros(maxCounts,1);
  wb_bit= zeros(maxCounts,1);
  wb_pump= zeros(maxCounts,1);
  wb_exit= zeros(maxCounts,1);
  wb_inject= zeros(maxCounts,1);
  africFactors = zeros(maxCounts,1);
  dfricFactors = zeros(maxCounts,1);
  
  

  
  
  
  
for timeCount = 1:maxCounts
 
u1 = timeCount/60000;
u7 = timeCount/60000;
  

    if(timeCount>=1 && timeCount<200)
        u1 = 1000/60000;
        u7 = 1000/60000;
    end
    if(timeCount>=200 && timeCount<300)
        u1 = (200+4*timeCount)/60000;
        u7 = 1000/60000;
    end
    if(timeCount>=300 && timeCount<500)
        u1 = 1400/60000;
        u7 = 1000/60000;
    end
    if(timeCount>=500 && timeCount<600)
        u1 = 1400/60000;
        u7 = (1000+3*timeCount-1500)/60000;
    end
    if(timeCount>=600 && timeCount<800)
        u1 = (1400)/60000;
        u7 = (1300)/60000;
    end
    if(timeCount>=800 && timeCount<900)
        u1 = (-200+2*timeCount)/60000;
        u7 = (1300)/60000;
    end
    if(timeCount>=900 && timeCount<1100)
        u1 = (1600)/60000;
        u7 = (1300)/60000;
    end
    if(timeCount>=1100 && timeCount<1300)
        u1 = (1600-8*(timeCount-1100))/60000;
        u7 = (1300-6.5*(timeCount-1100))/60000;
    end
    if(timeCount>=1300 && timeCount<1400)
        u1 = (0)/60000;
        u7 = (0)/60000;
    end
    if(timeCount>=1400 && timeCount<1600)
        u1 = (4*(timeCount-1400))/60000;
        u7 = (0)/60000;
    end
    if(timeCount>=1600 && timeCount<1700)
        u1 = (800+2*(timeCount-1600))/60000;
        u7 = (0)/60000;
    end
    if(timeCount>=1700 && timeCount<1900)
        u1 = (1000)/60000;
        u7 = (0)/60000;
    end
    if(timeCount>=1900 && timeCount<2100)
        u1 = (1000+2*(timeCount-1900))/60000;
        u7 = (3*(timeCount-1900))/60000;
    end
    if(timeCount>=2100 && timeCount<2500)
        u1 = (1400)/60000;
        u7 = (600)/60000;
    end
    if(timeCount>=2500 && timeCount<2700)
        u1 = (1400)/60000;
        u7 = (600+4*(timeCount-2500))/60000;
    end
    if(timeCount>=2700 && timeCount<=maxCounts)
        u1 = (1400)/60000;
        u7 = (1400)/60000;
    end    
    
  
      

  
  % Store controlled variables
  u1Array(timeCount) = u1;
  u2Array(timeCount) = u2;
  u3Array(timeCount) = u3;
  u4Array(timeCount) = u4;
  u5Array(timeCount) = u5;
  u6Array(timeCount) = u6;
  u7Array(timeCount) = u7;
  u8Array(timeCount) = u8;
  u9Array(timeCount) = u9;
  u10Array(timeCount) = u10;
  
  q_in(timeCount) = u1Array(timeCount);
  q_out(timeCount) = u7Array(timeCount);

      
 
  
  u = [u1,u2,u3,u4, u5, u6, u7, u8, u9, 10];
  U(:,timeCount) = u; 
  


  calllib('WeModForMatlab', 'setMainPumpVolumeRate',u1);
  calllib('WeModForMatlab', 'setChokeValveOpening',u2);
  calllib('WeModForMatlab', 'setChokePumpVolumeRate',u3);
  calllib('WeModForMatlab', 'setMainPumpDensity',u4);
  calllib('WeModForMatlab', 'setDrillstringVelocity',u5);
  calllib('WeModForMatlab', 'setChokePumpDensity',u6);
  calllib('WeModForMatlab', 'setAnnulusExtractionPumpVolumeRate',u7);
  calllib('WeModForMatlab', 'setAnnulusInjectionPumpVolumeRate',u8);
  calllib('WeModForMatlab', 'setAnnulusInjectionPumpDensity',u9);
  calllib('WeModForMatlab', 'setRotationRate',u10);
  

  maxVals = calllib('WeModForMatlab', 'step');
  y = zeros(maxVals,1);
  % Reading sensor values using get methods
   for yPos = 1:maxVals
    y(yPos)  = calllib('WeModForMatlab', 'getMeasurementUsingPos',yPos-1);% C array indexes start at 0
   end
  

   
  %Friction parameters
  africFactors(timeCount) = calllib('WeModForMatlab', 'getAnnulusFrictionFactorParameter');
  dfricFactors(timeCount)  = calllib('WeModForMatlab', 'getDrillStringFrictionFactorParameter');
   

   
  %Replace with point data structure
  spp(timeCount) = calllib('WeModForMatlab', 'getSPP');  %Stand pipe pressure
  bitp(timeCount) = calllib('WeModForMatlab', 'getPressureAfterBit');   %Bottom hole pressure in annulus
  cdp(timeCount) = calllib('WeModForMatlab', 'getChokePressure');   %Choke differential pressure
  
  bhp(timeCount) = y(26);
  
  BHP = bhp(timeCount)/1e5;
     

        
%    LLString(timeCount) = calllib('WeModForMatlab', 'getLiquidLevelString')
%    LLAnnulus(timeCount) = calllib('WeModForMatlab', 'getLiquidLevelAnnulus');
        
  
%     rTop(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',59);
%     pTop(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',49);
  
   riserBottom(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',54);

 %mud
  wm_bit(timeCount) =  -y(36); %Mass flow rate at bit
  wm_pump(timeCount) = y(46); % at pump
  wm_exit(timeCount) = -y(31); %and at exit
  wm_inject(timeCount) = -y(61); %and at injection pipe

%baryte
  wb_bit(timeCount) = -y(37); %Mass flow rate at bit
  wb_pump(timeCount) = y(47); % at pump
  wb_exit(timeCount) = -y(32); %and at exit
  wb_inject(timeCount) = -y(62); %and at injection pipe

%   for sensorPos = 1:numSensors
%     Y(timeCount,sensorPos,:) = y(1+(8*numBranches)+9*(sensorPos-1)+[1 2 8 9]);
%   end




%%%%%%%%%%%%%%%%
%Kalman filter %
%%%%%%%%%%%%%%%%
  
  freq = 30; % Timesteps between each bottom hole pressure update
  steps = 50; % Number of updates of the kalman filter between each timestep
  
  
  if(timeCount>1)

     
      if(timeCount<=freq)    
       
          for k=1:steps
              
              if(k==1)
                  
                  % Kalman filter algorithm updating on previous states and
                  % measurements
                
                  PK = F * P(:,:,timeCount-1) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-1) + (1/A * (u1Array(timeCount) - u7Array(timeCount)))/steps; xp(2,timeCount-1); xp(3,timeCount-1); xp(4,timeCount-1); xp(5,timeCount-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0 0 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) 1 u1Array(timeCount) u1Array(timeCount)^2];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) + x_K(4,k) * u1Array(timeCount) + x_K(5,k) * u1Array(timeCount)^2];
                   
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(5) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) + xpK(4,k) * u1Array(timeCount) + xpK(5,k) * u1Array(timeCount)^2];

                  
              elseif(k>1)
                  
                  % Kalman filter algorithm updating on states and
                  % measurements estimated in the same timestep, updating
                  % them several times per timestep to test if this gives
                  % better results
                
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount) - u7Array(timeCount)))/steps; xpK(2,k-1); xpK(3,k-1); xpK(4,k-1); xpK(5,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0 0 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) 1 u1Array(timeCount) u1Array(timeCount)^2];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) + x_K(4,k) * u1Array(timeCount) + x_K(5,k) * u1Array(timeCount)^2];
                  
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(5) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) + xpK(4,k) * u1Array(timeCount) + xpK(5,k) * u1Array(timeCount)^2];

              end
          end
                
          
          % The final values to use in the next timestep is set, using the
          % last update for the current timestep 
                
            P(:,:,timeCount) = PK;
            x_(:,timeCount) = x_K(:,steps);

            H(:,:,timeCount) = HK;
            K(:,:,timeCount) = KK;

            z_(:,timeCount) = z_K(:,steps);
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xpK(:,steps);
            P(:,:,timeCount) = PK;

            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) + xp(4,timeCount) * u1Array(timeCount) + xp(5,timeCount) * u1Array(timeCount)^2];

        
      else
        
        if(floor((timeCount-1)/freq) == (timeCount-1)/freq && timeCount>=freq) %Checking if the bottom hole pressure is to be updated

            bhpUpdate(timeCount) = bhp(timeCount-freq); % The measurement recieved at current timestep is the measurement from 'freq' time ago
            
            for k=1:steps
                
                if(k==1)
                
                    
                    % Kalman filter algorithm updating on previous states and
                  % measurements
                  
                  
                  PK = F * P(:,:,timeCount-freq) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-freq) + (1/A * (u1Array(timeCount-freq) - u7Array(timeCount-freq)))/steps; xp(2,timeCount-freq); xp(3,timeCount-freq); xp(4,timeCount-freq); xp(5,timeCount-freq)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0 0 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) 1 u1Array(timeCount) u1Array(timeCount)^2];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) + x_K(4,k) * u1Array(timeCount) + x_K(5,k) * u1Array(timeCount)^2];
                   
                  measK(:,k) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(5) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) + xpK(4,k) * u1Array(timeCount) + xpK(5,k) * u1Array(timeCount)^2];
                  
                elseif(k>1)
                    
                  % Kalman filter algorithm updating on states and
                  % measurements estimated in the same timestep, updating
                  % them several times per timestep to test if this gives
                  % better results
                    
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount-freq) - u7Array(timeCount-freq)))/steps; xpK(2,k-1); xpK(3,k-1); xpK(4,k-1); xpK(5,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0 0 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) 1 u1Array(timeCount) u1Array(timeCount)^2];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) + x_K(4,k) * u1Array(timeCount) + x_K(5,k) * u1Array(timeCount)^2];
                  
                  measK(:,k) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(5) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) + xpK(4,k) * u1Array(timeCount) + xpK(5,k) * u1Array(timeCount)^2];
                end
                
               

                PI(:,:,timeCount-freq) = PK;
                x_I(:,timeCount-freq) = x_K(:,steps);
                HI(:,:,timeCount-freq) = HK;
                KI(:,:,timeCount-freq) = KK;

                z_I(:,timeCount-freq) = z_K(:,steps);

                measI(:,timeCount-freq) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                xpI(:,timeCount-freq) = xpK(:,steps);
                PI(:,:,timeCount-freq) = PK;
            end


                for j=(timeCount-freq+1):(timeCount)
                    
                    %The current states and measurement estimates are
                    %estimated based on the delayed measurement
                PI(:,:,j) = F * PI(:,:,j-1) * F' + L * Q * L';
                x_I(:,j) = [xpI(1,j-1) + 1/A * (q_in(j) - q_out(j)); xpI(2,j-1); xpI(3,j-1); xpI(4,j-1); xpI(5,j-1)];

                HI(:,:,j) = [(x_I(2,j)-rho_w) * g g * x_I(1,j) 0 0 0; (x_I(2,j)-rho_w) * g g * x_I(1,j) + g * (d - h_r) 1 u1Array(timeCount) u1Array(timeCount)^2];
                KI(:,:,j) = PI(:,:,j) * HI(:,:,j)' * inv(HI(:,:,j) * PI(:,:,j) * HI(:,:,j)' + M * R + M');

                z_I(:,j) = [rho_w * g * (h_r - x_I(1,j)) + x_I(2,j) * g * x_I(1,j) + p0; rho_w * g * (h_r - x_I(1,j)) + x_I(2,j) * g * x_I(1,j) + p0 + x_I(2,j) * g * (d - h_r) + x_I(3,j) + x_I(4,j) * u1Array(j) + x_I(5,j) * u1Array(j)^2];

                measI(:,j) = [riserBottom(j);z_I(2,j)];

                xpI(:,j) = x_I(:,j) + KI(:,:,j) * (measI(:,j) - z_I(:,j));
                zpI(:,j) = [rho_w * g * (h_r - xpI(1,j)) + xpI(2,j) * g * xpI(1,j) + p0; rho_w * g * (h_r - xpI(1,j)) + xpI(2,j) * g * xpI(1,j) + p0 + xpI(2,j) * g * (d - h_r) + xpI(3,j) + xpI(4,j) * u1Array(j) + xpI(5,j) * u1Array(j)^2];

                PI(:,:,j) = (eye(5) - KI(:,:,j) * HI(:,:,j)) * PI(:,:,j);
                end

            %The new values for further simulation is the state and
            %measurement estimates estimated for the current timestep based
            %on the delayed measurement
                
            P(:,:,timeCount) = PI(:,:,timeCount);
            x_(:,timeCount) = x_I(:,timeCount);

            H(:,:,timeCount) = HI(:,:,timeCount);
            K(:,:,timeCount) = KI(:,:,timeCount);

            z_(:,timeCount) = z_I(:,timeCount);
            
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xpI(:,timeCount);
            
            zp(:,timeCount) = zpI(:,timeCount);
            


        else
            
            %Kalman filter algorithm performed for timesteps when the
            %measurement is not avaliable, estimating new states and
            %measurements based on estimate of bottom hole pressure
            
            P(:,:,timeCount) = F * P(:,:,timeCount-1) * F' + L * Q * L';
            x_(:,timeCount) = [xp(1,timeCount-1) + 1/A * (u1Array(timeCount) - u7Array(timeCount)); xp(2,timeCount-1); xp(3,timeCount-1); xp(4,timeCount-1); xp(5,timeCount-1)];

            H(:,:,timeCount) = [(x_(2,timeCount)-rho_w) * g g * x_(1,timeCount) 0 0 0; (x_(2,timeCount)-rho_w) * g g * x_(1,timeCount) + g * (d - h_r) 1 u1Array(timeCount) u1Array(timeCount)^2];
            K(:,:,timeCount) = P(:,:,timeCount) * H(:,:,timeCount)' * inv(H(:,:,timeCount) * P(:,:,timeCount) * H(:,:,timeCount)' + M * R + M');

            z_(:,timeCount) = [rho_w * g * (h_r - x_(1,timeCount)) + x_(2,timeCount) * g * x_(1,timeCount) + p0; rho_w * g * (h_r - x_(1,timeCount)) + x_(2,timeCount) * g * x_(1,timeCount) + p0 + x_(2,timeCount) * g * (d - h_r) + x_(3,timeCount) + x_(4,timeCount-1) * u1Array(timeCount) + x_(5,timeCount) * u1Array(timeCount)^2];
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = x_(:,timeCount) + K(:,:,timeCount) * (meas(:,timeCount) - z_(:,timeCount));
            P(:,:,timeCount) = (eye(5) - K(:,:,timeCount) * H(:,:,timeCount)) * P(:,:,timeCount);
            
            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) + xp(4,timeCount) * u1Array(timeCount) + xp(5,timeCount) * u1Array(timeCount)^2];

        end
        
      end
      
      
  end

%%%%%%%%%%%%%%%%%%%%%%%%
% End of kalman filter %
%%%%%%%%%%%%%%%%%%%%%%%%




  %Show progress
  if rem(timeCount,floor(100/timeStep)) == 0
    disp(sprintf('Time: %d [s] and BHP: %10.5f [bar]',floor(timeCount*timeStep),bhp(timeCount)/1e5));
  end
end
calllib('WeModForMatlab', 'terminateWeMod',caseNumber); 

disp('Simulation finished... unloading WeModForMatlab.dll..');
%whos



%save(filename,'U','spp','bhp','cdp','Y');
save(filename,'U','spp','bhp','cdp');
unloadlibrary('WeModForMatlab');
disp('Unloaded WeModForMatlab.dll, and plotting...');



%Different plots

figure
plot(bhpUpdate/1e5,'.b')
hold on
plot(bhp/1e5,'--r')
plot(zp(2,:)/1e5,'--g')
% plot(z(2,:)/1e5,'--k')
legend('update','real','est')
axis([0 3000 1650 1700])

figure
subplot(2,1,1)
plot(zp(2,:)/1e5-bhp'/1e5)
xlabel('Time [s]')
ylabel('Pressure [bar]')
axis([0 maxCounts -15 10])
grid on
title('The error between the calculated and the measured bottom hole pressure in the well')
subplot(2,1,2)
plot(zp(1,:)/1e5-riserBottom/1e5)
grid on
axis([0 maxCounts -0.1 0.1])
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('The error between the calculated and the measured pressure at the extraction pump')

figure
subplot(2,1,1)
plot(zp(2,:)/1e5)
hold on
plot(bhp/1e5,'--r')
xlabel('Time [s]')
ylabel('Pressure [bar]')
axis([0 maxCounts 1660 1690])
legend('Calculated','Measured')
title('The bottom hole pressure in the well')
grid on
subplot(2,1,2)
grid on
plot(zp(1,:)/1e5)
hold on
plot(riserBottom/1e5,'--r')
axis([0 maxCounts 380 386])
grid on
legend('Calculated','Measured')
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('The pressure at the extraction pump')

figure
plot(u1Array*60000)
hold on
plot(u7Array*60000,'r')
plot((u1Array-u7Array)*60000,'g')
legend('q_{in}','q_{out}','q_{in}-q_{out}')
xlabel('Time [s]')
ylabel('Flow [l/min]')
title('The flow in the system')
grid on

figure
subplot(2,1,1)
plot(xp(1,:))
xlabel('Time [s]')
ylabel('Height [m]')
title('Height of mud in the riser')
axis([0 maxCounts 2000 2130])
grid on
subplot(2,1,2)
plot(xp(2,:))
xlabel('Time [s]')
ylabel('Weight [kg/m^3]')
title('Density of the mud')
axis([0 maxCounts 1820 1875])
grid on


