% function simulateUsingR02(maxCounts,filename,plotDimension)
% Calculates drilling fluid flowrates and pressures
% (c) IRIS Petroleum
% 1. October 2008 GNY
% 12. Nov 2008 OEBR (oebr@iris.no)

clear all



filename = 'test1'; 

%Constants

g = 9.81;
d = 9220;
h_r = 2150;
rho_w = 0;
riserID = 0.473;
dsOD = 0.127;
A = pi*riserID^2 - pi*dsOD^2;
p0 = 1;

maxCounts = 3000; %Simulation length

%Initial values
xp(:,1) = [2100;1850;1.13e8];
xpI(:,1) = xp(:,1);
xp2(:,1) = xp(:,1);
x_(:,1) = xp(:,1);
x_2(:,1) = xp(:,1);


P(:,:,1) = eye(3); %Error covariance matrix

%Jacobian matrices
F = zeros(3);
L = eye(3);
M = eye(2);


%Noise
sigmaw1 = 0.001;
sigmaw2 = 0.001;
sigmaw3 = 0.001;
sigmav1 = 30;
sigmav2 = 50;

%Noise covariance matrices
Q = zeros(3);
Q(1,1) = 10000*sigmaw1^2;
Q(2,2) = 10000*sigmaw2^2;
Q(3,3) = 10000*sigmaw3^2;
R = zeros(2);
R(1,1) = 10000*sigmav1^2;
R(2,2) = 10000*sigmav2^2;


%First timestep

u1Array(1) = 1000/60000;
u7Array(1) = 1000/60000;
u8Array(1) = 0;

z_(:,1) = [rho_w * g * (h_r - x_(1,1)) + x_(2,1) * g * x_(1,1) + p0; rho_w * g * (h_r - x_(1,1)) + x_(2,1) * g * x_(1,1) + p0 + x_(2,1) * g * (d - h_r) + x_(3,1) * u1Array(1)];
zp(:,1) = [rho_w * g * (h_r - xp(1,1)) + xp(2,1) * g * xp(1,1) + p0; rho_w * g * (h_r - xp(1,1)) + xp(2,1) * g * xp(1,1) + p0 + xp(2,1) * g * (d - h_r) + xp(3,1) * u1Array(1)];
zpK(:,1) = zp(:,1);
meas(:,1) = [z_(1,1); z_(2,1)];


  % Brukes midlertidig til Main Pump
  firstStartUp = 0;
  firstStartUpTime = 1*60;
  firstFlow    = 1000/(60*1000);
  startFirstCounter     = 0;

  
  % Brukes midlertidig til Extraction Pump
  fourthStartUp = 0;
  fourthStartUpTime = 1*60;
  fourthFlow    = 0/(60*1000);
  startFourthCounter     = 0;
  
  
  u7_old = fourthFlow;%1400/(60*1000);




timeStep = 1; % 1 second per timeStep


caseNumber = 8; 
initWeMod(caseNumber);
    
% Defining initial u vector
% u1 - main pump rate, [m3/s] 
% u2 - choke opening, [m2]
% u3 - choke pump annulus pump rate, [m3/s] 
% u4 - fluid density, [kg/m3](alternative: [SG] (relative to water density 1)
% u5 - drillstring velocity  [m/s]
% u6 - choke pump fluid density  [kg/m3]
% u7 - annulus extraction pump flow rate  [m3/s]
% u8 - annulus injection pump flow rate  [m3/s]  
% u9 - annulus injection pump density  [kg/m3]
% u10 - Drillstring Rotation. [RPM]

u1 = 0;
u2 = 1; % m2, more than full opening...
u3 = 0; %Not available in this version...
u4 = 1759.7;
u5 = 0;
u6 = 1000;
u7 = 0;
u8 = 0;
u9 = 1760;
u10 = 0;




% DEFINE REFERENCE VALUES HERE:
% reference pressures, annulus velocities etc...
%
% end of reference definitions

% initialize controller
    
% end of initialize controller

% initialize observer

%end of initialize observer

%Define sensor selection
%numSensors = 12; % each having 9 values (pres + 7*flow + temp)
%numBranches = 5; % each having 8 values (pres + 7*flow)
%Define sensor selection
numSensors = 0; % each having 9 values (pres + 7*flow + temp)
numBranches = 4; % each having 8 values (pres + 7*flow)

% Extract sensor names
numSensorNames = calllib('WeModForMatlab','getSensorNameListSize');
% numSensorNames
for SensorNum = 1:numSensorNames
    SensorName = calllib('WeModForMatlab', 'getSensorName',SensorNum-1);
    disp(SensorName)
    save('namefile','SensorName');
end


%loop
disp(sprintf('Simulating to %d [s]',maxCounts));

u1Array = zeros(maxCounts,1);
u2Array = zeros(maxCounts,1);
u3Array = zeros(maxCounts,1);
u4Array = zeros(maxCounts,1);
u5Array = zeros(maxCounts,1);
u6Array = zeros(maxCounts,1);
u7Array = zeros(maxCounts,1);
u8Array = zeros(maxCounts,1);
u9Array = zeros(maxCounts,1);
u10Array = zeros(maxCounts,1);

U = zeros(10,maxCounts);
  spp = zeros(maxCounts,1);
  bhp= zeros(maxCounts,1);
  cdp= zeros(maxCounts,1);
 
  wm_bit= zeros(maxCounts,1);
  wm_pump= zeros(maxCounts,1);
  wm_exit= zeros(maxCounts,1);
  wm_inject= zeros(maxCounts,1);
  wb_bit= zeros(maxCounts,1);
  wb_pump= zeros(maxCounts,1);
  wb_exit= zeros(maxCounts,1);
  wb_inject= zeros(maxCounts,1);
  africFactors = zeros(maxCounts,1);
  dfricFactors = zeros(maxCounts,1);
  
for timeCount = 1:maxCounts
  
    %Flow in the system
    
%     if(timeCount>=1 && timeCount<200)
%         u1 = 1000/60000;
%         u8 = 0/60000;
%         u7 = 1000/60000;
%     end
%     if(timeCount>=200 && timeCount<300)
%         u1 = (200+4*timeCount)/60000;
%         u8 = 0/60000;
%         u7 = (1000+4*(timeCount-200))/60000;
%     end
%     if(timeCount>=300 && timeCount<500)
%         u1 = 1400/60000;
%         u8 = 5*(timeCount-300)/60000;
%         u7 = 1400/60000;
%     end
%     if(timeCount>=500 && timeCount<600)
%         u1 = 1400/60000;
%         u8 = 1000/60000;
%         u7 = (1400+3*(timeCount-500))/60000;
%     end
%     if(timeCount>=600 && timeCount<800)
%         u1 = (1400)/60000;
%         u8 = 1000/60000;
%         u7 = (1700)/60000;
%     end
%     if(timeCount>=800 && timeCount<900)
%         u1 = (-200+2*timeCount)/60000;
%         u8 = (1000-4*(timeCount-800))/60000;
%         u7 = (1700)/60000;
%     end
%     if(timeCount>=900 && timeCount<1100)
%         u1 = (1600)/60000;
%         u8 = (600)/60000;
%         u7 = (1700)/60000;
%     end
%     if(timeCount>=1100 && timeCount<1300)
%         u1 = (1600-8*(timeCount-1100))/60000;
%         u8 = (600-3*(timeCount-1100))/60000;
%         u7 = (1700-8.5*(timeCount-1100))/60000;
%     end
%     if(timeCount>=1300 && timeCount<1400)
%         u1 = (0)/60000;
%         u8 = 0/60000;
%         u7 = (0)/60000;
%     end
%     if(timeCount>=1400 && timeCount<1600)
%         u1 = (4*(timeCount-1400))/60000;
%         u8 = 0/60000;
%         u7 = (0)/60000;
%     end
%     if(timeCount>=1600 && timeCount<1700)
%         u1 = (800+2*(timeCount-1600))/60000;
%         u8 = 0/60000;
%         u7 = (0)/60000;
%     end
%     if(timeCount>=1700 && timeCount<1900)
%         u1 = (1000)/60000;
%         u8 = 0/60000;
%         u7 = (0)/60000;
%     end
%     if(timeCount>=1900 && timeCount<2100)
%         u1 = (1000+2*(timeCount-1900))/60000;
%         u8 = 0/60000;
%         u7 = (5*(timeCount-1900))/60000;
%     end
%     if(timeCount>=2100 && timeCount<2500)
%         u1 = (1400)/60000;
%         u8 = (timeCount-2100)/60000;
%         u7 = (1000)/60000;
%     end
%     if(timeCount>=2500 && timeCount<2700)
%         u1 = (1400)/60000;
%         u8 = 400/60000;
%         u7 = (1000+4*(timeCount-2500))/60000;
%     end
%     if(timeCount>=2700 && timeCount<=maxCounts)
%         u1 = (1400)/60000;
%         u8 = 400/60000;
%         u7 = (1800)/60000;
%     end  

% if(timeCount>=1 && timeCount<=600)
%     u1 = 1000/60000;
%     u7 = 0/60000;
%     u8=0;
% end
% if(timeCount>=600 && timeCount<=800)
%     u1 = (1000-5*(timeCount-600))/60000;
%     u7 = (5*(timeCount-600))/60000;
%     u8=0;
% end
% if(timeCount>=800 && timeCount<=1400)
%     u1 = 0/60000;
%     u7 = 1000/60000;
%     u8=0;
% end
% if(timeCount>=1400 && timeCount<=1600)
%     u1 = 5*(timeCount-1400)/60000;
%     u7 = (1000-5*(timeCount-1400))/60000;
%     u8=0;
% end
% if(timeCount>=1600 && timeCount<=2200)
%     u1 = 1000/60000;
%     u7 = 0/60000;
%     u8=0;
% end
% if(timeCount>=2200 && timeCount<=2400)
%     u1 = (1000-5*(timeCount-2200))/60000;
%     u7 = (5*(timeCount-2200))/60000;
%     u8=0;
% end
% if(timeCount>=2400 && timeCount<=3000)
%     u1 = 0/60000;
%     u7 = 1000/60000;
%     u8=0;
% end
% if(timeCount>=3000 && timeCount<=3200)
%     u1 = 5*(timeCount-3000)/60000;
%     u7 = (1000-5*(timeCount-3000))/60000;
%     u8=0;
% end
% if(timeCount>=3200 && timeCount<=3800)
%     u1 = 1000/60000;
%     u7 = 0/60000;
%     u8=0;
% end
% if(timeCount>=3800 && timeCount<=4000)
%     u1 = (1000-5*(timeCount-3800))/60000;
%     u7 = (5*(timeCount-3800))/60000;
%     u8=0;
% end
% if(timeCount>=4000 && timeCount<=4600)
%     u1 = 0/60000;
%     u7 = 1000/60000;
%     u8=0;
% end
% if(timeCount>=4600 && timeCount<=4800)
%     u1 = 5*(timeCount-4600)/60000;
%     u7 = (1000-5*(timeCount-4600))/60000;
%     u8=0;
% end
% if(timeCount>=4800 && timeCount<=maxCounts)
%     u1 = 1000/60000;
%     u7 = 0/60000;
%     u8=0;
% end

%

    if(timeCount>=1 && timeCount<500)
        u1 = 1000/60000;
        u8 = 500/60000;
        u7 = 1500/60000;
    end
    if(timeCount>=500 && timeCount<1000)
        u1 = (1000+500*sin(2*pi*(timeCount-500)/2000))/60000;
        u8 = (500-500*sin(2*pi*(timeCount-500)/2000))/60000;
        u7 = (1500+500*sin(2*pi*(timeCount-500)/2000))/60000;
    end
    if(timeCount>=1000 && timeCount<1300)
        u1 = 1500/60000;
        u8 = 0/60000;
        u7 = 2000/60000;
    end
    if(timeCount>=1300 && timeCount<1700)
        u1 = (1500+400*sin(2*pi*(timeCount-1300)/1600))/60000;
        u8 = (500*sin(2*pi*(timeCount-1300)/1600))/60000;
        u7 = (2000-3*(timeCount-1300))/60000;
    end
    if(timeCount>=1700 && timeCount<2200)
        u1 = (1900)/60000;
        u8 = 500/60000;
        u7 = (800)/60000;
    end
    if(timeCount>=2200 && timeCount<2600)
        u1 = (1900)/60000;
        u8 = (500-500*sin(2*pi*(timeCount-2200)/1600))/60000;
        u7 = (800+800*sin(2*pi*(timeCount-2200)/1600))/60000;
    end
    if(timeCount>=2600 && timeCount<=maxCounts)
        u1 = (1900)/60000;
        u8 = (0)/60000;
        u7 = (1600)/60000;
    end


  % Store controlled variables
  u1Array(timeCount) = u1;
  u2Array(timeCount) = u2;
  u3Array(timeCount) = u3;
  u4Array(timeCount) = u4;
  u5Array(timeCount) = u5;
  u6Array(timeCount) = u6;
  u7Array(timeCount) = u7;
  u8Array(timeCount) = u8;
  u9Array(timeCount) = u9;
  u10Array(timeCount) = u10;
 
  
  u = [u1,u2,u3,u4, u5, u6, u7, u8, u9, 10];
  U(:,timeCount) = u; 
    

[depthProfileAnnulus,...
          verticalDepthProfileAnnulus,...
          pressureProfileAnnulus,...
          velocityProfileAnnulus,...
          densityProfileAnnulus,...
          temperatureProfileAnnulus,...
          massProfileAnnulus,...
          volumeProfileAnnulus,...
          depthProfileDrillString,...
          pressureProfileDrillString,...
          velocityProfileDrillString,...
          maxVals] = stepWeMod(u);
  
%   depthProfileMatrixAnnulus(:,timeCount)  = depthProfileAnnulus;    
%   verticalDepthProfileMatrixAnnulus(:,timeCount)  =  verticalDepthProfileAnnulus;
%   pressureProfileMatrixAnnulus(:,timeCount)  = pressureProfileAnnulus;
%   velocityProfileMatrixAnnulus(:,timeCount)  = velocityProfileAnnulus;
%   densityProfileMatrixAnnulus(:,timeCount)  = densityProfileAnnulus;
%   temperatureProfileMatrixAnnulus(:,timeCount)  = temperatureProfileAnnulus;
%   massProfileMatrixAnnulus(:,timeCount)  = massProfileAnnulus;
%   volumeProfileMatrixAnnulus(:,timeCount)  = volumeProfileAnnulus;
% 
%   depthProfileMatrixDrillString(:,timeCount)  = depthProfileDrillString;    
%   pressureProfileMatrixDrillString(:,timeCount)  = pressureProfileDrillString;
%   velocityProfileMatrixDrillString(:,timeCount)  = velocityProfileDrillString;
  
  y = zeros(maxVals,1);
  % Reading sensor values using get methods
   for yPos = 1:maxVals
    y(yPos)  = calllib('WeModForMatlab', 'getMeasurementUsingPos',yPos-1);% C array indexes start at 0
   end
  
 
  %Friction parameters
  africFactors(timeCount) = calllib('WeModForMatlab', 'getAnnulusFrictionFactorParameter');
  dfricFactors(timeCount)  = calllib('WeModForMatlab', 'getDrillStringFrictionFactorParameter');
   
   
  %Replace with point data structure
  spp(timeCount) = calllib('WeModForMatlab', 'getSPP');  %Stand pipe pressure
  bhp(timeCount) = calllib('WeModForMatlab', 'getPressureAfterBit');   %Bottom hole pressure in annulus
  cdp(timeCount) = calllib('WeModForMatlab', 'getChokePressure');   %Choke differential pressure
  
  BHP = bhp(timeCount)/1e5;
     
       
   rTop(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',50);
   pTop(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',40);
  

  riserBottom(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',45);

%  mud
  wm_bit(timeCount) =  -y(32);%-y(36); %Mass flow rate at bit
  wm_pump(timeCount) = y(37);%y(46); % at pump
  wm_exit(timeCount) =  -y(27);%-y(31); %and at exit
  wm_inject(timeCount) =  -y(52);%-y(61); %and at injection pipe

%baryte
  wb_bit(timeCount) = -y(33);%-y(37); %Mass flow rate at bit
  wb_pump(timeCount) = y(38);%y(47); % at pump
  wb_exit(timeCount) = -y(28);%-y(32); %and at exit
  wb_inject(timeCount) = -y(53);%-y(62); %and at injection pipe


  
  % ADD OBSERVER CODE HERE
  
  freq = 30; %Time between bhp measurements arrive
  steps = 1; %How many times the filter is updated for each timestep
  
  if(timeCount>1)
      
     
      if(timeCount<=freq)    
       
          for k=1:steps
              
              if(k==1)
                
                  PK = F * P(:,:,timeCount-1) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-1) + (1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)))/steps; xp(2,timeCount-1); xp(3,timeCount-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
                   
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];

                  
              elseif(k>1)
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)))/steps; xpK(2,k-1); xpK(3,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
                  
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];

              end
          end
                
                
            P(:,:,timeCount) = PK;
            x_(:,timeCount) = x_K(:,steps);

            H(:,:,timeCount) = HK;
            K(:,:,timeCount) = KK;

            z_(:,timeCount) = z_K(:,steps);
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xpK(:,steps);
            P(:,:,timeCount) = PK;

            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) * u1Array(timeCount)];

        
      else
        
        if(floor((timeCount-1)/freq) == (timeCount-1)/freq && timeCount>=freq)
            bhpUpdate(timeCount) = bhp(timeCount-freq);
            
            for k=1:steps
                
                if(k==1)
                
                  PK = F * P(:,:,timeCount-freq) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-freq) + (1/A * (u1Array(timeCount-freq) + u8Array(timeCount-freq) - u7Array(timeCount-freq)))/steps; xp(2,timeCount-freq); xp(3,timeCount-freq)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount-freq)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount-freq)];
                   
                  measK(:,k) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];
                  
                elseif(k>1)
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount-freq) + u8Array(timeCount-freq) - u7Array(timeCount-freq)))/steps; xpK(2,k-1); xpK(3,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount-freq)];
                  
                  measK(:,k) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];
                end

                PI(:,:,timeCount-freq) = PK;
                x_I(:,timeCount-freq) = x_K(:,steps);
                HI(:,:,timeCount-freq) = HK;
                KI(:,:,timeCount-freq) = KK;

                z_I(:,timeCount-freq) = z_K(:,steps);

                measI(:,timeCount-freq) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                xpI(:,timeCount-freq) = xpK(:,steps);
                PI(:,:,timeCount-freq) = PK;
            end


                for j=(timeCount-freq+1):(timeCount)
                PI(:,:,j) = F * PI(:,:,j-1) * F' + L * Q * L';
                x_I(:,j) = [xpI(1,j-1) + 1/A * (u1Array(j) + u8Array(j) - u7Array(j)); xpI(2,j-1); xpI(3,j-1)];

                HI(:,:,j) = [(x_I(2,j)-rho_w) * g g * x_I(1,j) 0; (x_I(2,j)-rho_w) * g g * x_I(1,j) + g * (d - h_r) u1Array(j)];
                KI(:,:,j) = PI(:,:,j) * HI(:,:,j)' * inv(HI(:,:,j) * PI(:,:,j) * HI(:,:,j)' + M * R + M');

                z_I(:,j) = [rho_w * g * (h_r - x_I(1,j)) + x_I(2,j) * g * x_I(1,j) + p0; rho_w * g * (h_r - x_I(1,j)) + x_I(2,j) * g * x_I(1,j) + p0 + x_I(2,j) * g * (d - h_r) + x_I(3,j) * u1Array(j)];

                measI(:,j) = [riserBottom(j);z_I(2,j)];

                xpI(:,j) = x_I(:,j) + KI(:,:,j) * (measI(:,j) - z_I(:,j));
                zpI(:,j) = [rho_w * g * (h_r - xpI(1,j)) + xpI(2,j) * g * xpI(1,j) + p0; rho_w * g * (h_r - xpI(1,j)) + xpI(2,j) * g * xpI(1,j) + p0 + xpI(2,j) * g * (d - h_r) + xpI(3,j) * u1Array(j)];

                PI(:,:,j) = (eye(3) - KI(:,:,j) * HI(:,:,j)) * PI(:,:,j);
            end

            P(:,:,timeCount) = PI(:,:,timeCount);
            x_(:,timeCount) = x_I(:,timeCount);

            H(:,:,timeCount) = HI(:,:,timeCount);
            K(:,:,timeCount) = KI(:,:,timeCount);

            z_(:,timeCount) = z_I(:,timeCount);
            
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xpI(:,timeCount);
            
            zp(:,timeCount) = zpI(:,timeCount);
            


        else
            
            P(:,:,timeCount) = F * P(:,:,timeCount-1) * F' + L * Q * L';
            x_(:,timeCount) = [xp(1,timeCount-1) + 1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)); xp(2,timeCount-1); xp(3,timeCount-1)];

            H(:,:,timeCount) = [(x_(2,timeCount)-rho_w) * g g * x_(1,timeCount) 0; (x_(2,timeCount)-rho_w) * g g * x_(1,timeCount) + g * (d - h_r) u1Array(timeCount)];
            K(:,:,timeCount) = P(:,:,timeCount) * H(:,:,timeCount)' * inv(H(:,:,timeCount) * P(:,:,timeCount) * H(:,:,timeCount)' + M * R + M');

            z_(:,timeCount) = [rho_w * g * (h_r - x_(1,timeCount)) + x_(2,timeCount) * g * x_(1,timeCount) + p0; rho_w * g * (h_r - x_(1,timeCount)) + x_(2,timeCount) * g * x_(1,timeCount) + p0 + x_(2,timeCount) * g * (d - h_r) + x_(3,timeCount) * u1Array(timeCount)];
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = x_(:,timeCount) + K(:,:,timeCount) * (meas(:,timeCount) - z_(:,timeCount));
            P(:,:,timeCount) = (eye(3) - K(:,:,timeCount) * H(:,:,timeCount)) * P(:,:,timeCount);
            
            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) * u1Array(timeCount)];

        end
        
      end
      
 
          
        if(timeCount>=1000 && timeCount<=2800) % Timestep for missing bhp measurement

          bhpUpdate(timeCount) = 0;
          
        
          for k=1:steps
              
              if(k==1)
                
                  PK = F * P(:,:,timeCount-1) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-1) + (1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)))/steps; xp(2,timeCount-1); xp(3,timeCount-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
                   
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];

                  
              elseif(k>1)
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)))/steps; xpK(2,k-1); xpK(3,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
                  
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];

              end
          end
                
                
            P(:,:,timeCount) = PK;
            x_(:,timeCount) = x_K(:,steps);

            H(:,:,timeCount) = HK;
            K(:,:,timeCount) = KK;

            z_(:,timeCount) = z_K(:,steps);
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xpK(:,steps);
            P(:,:,timeCount) = PK;

            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) * u1Array(timeCount)];

        
          
      end
      

% %        if(timeCount>=300 && timeCount<=600 || timeCount>=2500 && timeCount<=2800)  %Timestep for missing riserbottom measurement
%           
%          
% 
%        
%           for k=1:steps
%               
%               if(k==1)
%                 
%                   PK = F * P(:,:,timeCount-1) * F' + L * Q * L';
%                   x_K(:,k) = [xp(1,timeCount-1) + (1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)))/steps; xp(2,timeCount-1); xp(3,timeCount-1)];
%                   HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
%                   KK = PK * HK' * inv(HK * PK * HK' + M * R + M');
% 
%                   z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
%                    
%                   measK(:,k) = [z_K(1,k);z_K(2,k)];
%                   xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
%                   PK = (eye(3) - KK * HK) * PK;
%                   zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];
% 
%                   
%               elseif(k>1)
%                   PK = F * PK * F' + L * Q * L';
%                   x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)))/steps; xpK(2,k-1); xpK(3,k-1)];
%                   HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
%                   KK = PK * HK' * inv(HK * PK * HK' + M * R + M');
% 
%                   z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
%                   
%                   measK(:,k) = [z_K(1,k);z_K(2,k)];
%                   xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
%                   PK = (eye(3) - KK * HK) * PK;
%                   zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];
% 
%               end
%           end
%                 
%                 
%             P(:,:,timeCount) = PK;
%             x_(:,timeCount) = x_K(:,steps);
% 
%             H(:,:,timeCount) = HK;
%             K(:,:,timeCount) = KK;
% 
%             z_(:,timeCount) = z_K(:,steps);
%             meas(:,timeCount) = [z_(1,timeCount);z_(2,timeCount)];
% 
%             xp(:,timeCount) = xpK(:,steps);
%             P(:,:,timeCount) = PK;
% 
%             zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) * u1Array(timeCount)];
% 
%         
%           
%       end

      
      
  end

  
  
  
  
  %
  % End of observer code

  % ADD Evaluation of reference here
  %
  % End of evaluating reference
  
  %Show progress
  if rem(timeCount,floor(100/timeStep)) == 0
    disp(sprintf('Time: %d [s] and BHP: %10.5f [bar]',floor(timeCount*timeStep),bhp(timeCount)/1e5));
  end
end

save(filename,'U','spp','bhp','cdp');

endWeMod(caseNumber);

%%
%Plots

figure
subplot(2,1,1)
plot(zp(2,:)/1e5)
hold on
plot(bhp/1e5,'--r')
plot(bhpUpdate/1e5,'.k')
xlabel('Time [s]')
ylabel('Pressure [bar]')
legend('Estimated','Measured','Updated measurements')
title('Bottom hole pressure')
grid on
axis([0 maxCounts 1640 1715])
subplot(2,1,2)
plot(zp(1,:)/1e5)
hold on
plot(riserBottom/1e5,'--r')
xlabel('Time [s]')
ylabel('Pressure [bar]')
legend('Estimated','Measured')
title('Pressure at extraction pump')
grid on
axis([0 maxCounts 365 388])


figure
subplot(2,1,1)
plot(zp(2,:)/1e5-bhp'/1e5)
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('Error between estimated and measured bottom hole pressure')
grid on
axis([0 maxCounts -3 2])
subplot(2,1,2)
plot(zp(1,:)/1e5-riserBottom/1e5)
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('Error between estimated and measured pressure at the extraction pump')
grid on
axis([0 maxCounts -0.3 0.3])

figure
subplot(3,1,1)
plot(xp(1,:))
xlabel('Time [s]')
ylabel('Height [m]')
title('Height of mud in the riser')
axis([0 maxCounts 2095 2115])
grid on
subplot(3,1,2)
plot(xp(2,:))
xlabel('Time [s]')
ylabel('Weight [kg/m^3]')
title('Density of the mud')
axis([0 maxCounts 1840 1865])
grid on
subplot(3,1,3)
plot(xp(3,:))
xlabel('Time [s]')
ylabel('Fa')
grid on
title('Friction factor')



figure
plot(u1Array*60000)
hold on
plot(u8Array*60000,'k')
plot(u7Array*60000,'r')
plot((u1Array+u8Array-u7Array)*60000,'g')
legend('q_{in}','q_{fill}','q_{out}','q_{in}+q_{fill}-q_{out}')
xlabel('Time [s]')
ylabel('Flow [l/min]')
grid on
title('The flow in the system')



plot(zp(2,:)/1e5)
hold on
plot(bhp/1e5,'--r')
plot(bhpUpdate/1e5,'.k')
xlabel('Time [s]')
ylabel('Pressure [bar]')
legend('Estimated','Measured','Updated measurements')
title('Bottom hole pressure')
grid on
axis([0 maxCounts 1640 1715])
subplot(2,1,2)
plot(zp(1,:)/1e5)
hold on
plot(riserBottom/1e5,'--r')
xlabel('Time [s]')
ylabel('Pressure [bar]')
legend('Estimated','Measured')
title('Pressure at extraction pump')
grid on
axis([0 maxCounts 365 388])
