% clear all; %close all;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The testing of the ORS system %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Constants
g = 9.81;
d = 9220;
h_r = 2150;
riserID = 0.473;
dsOD = 0.127;
A = pi*riserID^2 - pi*dsOD^2;
p0 = 1e5;

%Simulationtime

maxCounts = 3000;



%Initial values

x(:,1) = [2020;1880;1.1e8];
xp(:,1) = [1900;1800;1.3e8];
x_(:,1) = xp(:,1);
xc(:,1) = x(:,1);


P = eye(3); %Error covariance matrix

%Jacobi matrices

F = zeros(3);
L = eye(3);
M = eye(2);

%Process and measurement noise

sigmaw1 = 0.08;
sigmaw2 = 0.08;
sigmaw3 = 1;
sigmav1 = 1000;
sigmav2 = 3000;

%Process noise covariance matrix

Q = zeros(3);
Q(1,1) = sigmaw1^2;
Q(2,2) = sigmaw2^2;
Q(3,3) = sigmaw3^2;

%Measurement noise covariance matrix

R = zeros(2);
R(1,1) = sigmav1^2;
R(2,2) = sigmav2^2;


%The first timestep


q_in(1) = 1000/60000;
q_f(1) = 0;
q_out(1) = 1000/60000;


z(:,1) = [x(2,1) * g * x(1,1) + p0 + randn * sigmav1; x(2,1) * g * x(1,1) + p0 + x(2,1) * g * (d - h_r) + x(3,1) * q_in(1)^2 + randn * sigmav2];
zc(:,1) = [xc(2,1) * g * xc(1,1) + p0; xc(2,1) * g * xc(1,1) + p0 + xc(2,1) * g * (d - h_r) + xc(3,1) * q_in(1)^2];
z_(:,1) = [x_(2,1) * g * x_(1,1) + p0; x_(2,1) * g * x_(1,1) + p0 + x_(2,1) * g * (d - h_r) + x_(3,1) * q_in(1)^2];
zp(:,1) = [xp(2,1) * g * xp(1,1) + p0; xp(2,1) * g * xp(1,1) + p0 + xp(2,1) * g * (d - h_r) + xp(3,1) * q_in(1)^2];
 


for k = 2:maxCounts
    
    
    %The flows in the system
    
    if(k>=1 && k<200)
        q_in(k) = 1000/60000;
        q_f(k) = 0/60000;
        q_out(k) = 1000/60000;
    end
    if(k>=200 && k<300)
        q_in(k) = (200+4*k)/60000;
        q_f(k) = 0/60000;
        q_out(k) = (1000+6*(k-200))/60000;
    end
    if(k>=300 && k<500)
        q_in(k) = 1400/60000;
        q_f(k) = 5*(k-300)/60000;
        q_out(k) = 1600/60000;
    end
    if(k>=500 && k<600)
        q_in(k) = 1400/60000;
        q_f(k) = 1000/60000;
        q_out(k) = (1600+3*(k-500))/60000;
    end
    if(k>=600 && k<800)
        q_in(k) = (1400)/60000;
        q_f(k) = 1000/60000;
        q_out(k) = (1900)/60000;
    end
    if(k>=800 && k<900)
        q_in(k) = (-200+2*k)/60000;
        q_f(k) = (1000-4*(k-800))/60000;
        q_out(k) = (1900)/60000;
    end
    if(k>=900 && k<1100)
        q_in(k) = (1600)/60000;
        q_f(k) = (600)/60000;
        q_out(k) = (1900)/60000;
    end
    if(k>=1100 && k<1300)
        q_in(k) = (1600-8*(k-1100))/60000;
        q_f(k) = (600-3*(k-1100))/60000;
        q_out(k) = (1900-9.5*(k-1100))/60000;
    end
    if(k>=1300 && k<1400)
        q_in(k) = (0)/60000;
        q_f(k) = 0/60000;
        q_out(k) = (0)/60000;
    end
    if(k>=1400 && k<1600)
        q_in(k) = (4*(k-1400))/60000;
        q_f(k) = 0/60000;
        q_out(k) = (0)/60000;
    end
    if(k>=1600 && k<1700)
        q_in(k) = (800+2*(k-1600))/60000;
        q_f(k) = 0/60000;
        q_out(k) = (0)/60000;
    end
    if(k>=1700 && k<1900)
        q_in(k) = (1000)/60000;
        q_f(k) = 0/60000;
        q_out(k) = (0)/60000;
    end
    if(k>=1900 && k<2100)
        q_in(k) = (1000+2*(k-1900))/60000;
        q_f(k) = 0/60000;
        q_out(k) = (5*(k-1900))/60000;
    end
    if(k>=2100 && k<2500)
        q_in(k) = (1400)/60000;
        q_f(k) = (k-2100)/60000;
        q_out(k) = (1000)/60000;
    end
    if(k>=2500 && k<2700)
        q_in(k) = (1400)/60000;
        q_f(k) = 400/60000;
        q_out(k) = (1000+4*(k-2500))/60000;
    end
    if(k>=2700 && k<=maxCounts)
        q_in(k) = (1400)/60000;
        q_f(k) = 400/60000;
        q_out(k) = (1800)/60000;
    end
    
    %Update of the real states and measurements
    
    x(:,k) = [x(1,k-1) + 1/A * (q_in(k) + q_f(k) - q_out(k)) + randn * sigmaw1; x(2,k-1) + randn * sigmaw2; x(3,k-1) + randn * sigmaw3];
    z(:,k) = [x(2,k) * g * x(1,k) + p0 + randn * sigmav1; x(2,k) * g * x(1,k) + p0 + x(2,k) * g * (d - h_r) + x(3,k) * q_in(k)^2 + randn * sigmav2];
    xc(:,k) = [xc(1,k-1) + 1/A * (q_in(k) + q_f(k) - q_out(k)); xc(2,k-1); xc(3,k-1)];
    zc(:,k) = [xc(2,k) * g * xc(1,k) + p0; xc(2,k) * g * xc(1,k) + p0 + xc(2,k) * g * (d - h_r) + xc(3,k) * q_in(k)^2];
  
    steps = 1; %The number of updates of the kalman filter between each measurement
    
    %Kalman filter algorithm
    
    for i=1:steps
        if(i==1)
            PI = F * P * F' + L * Q * L';
            x_I(:,i) = [xp(1,k-1) + (1/A * (q_in(k) + q_f(k) - q_out(k)))/steps; xp(2,k-1); xp(3,k-1)];
            HI = [(x_I(2,i)) * g g * x_I(1,i) 0; (x_I(2,i)) * g g * x_I(1,i) + g * (d - h_r) q_in(k)^2];
            KI = PI * HI' * inv(HI * PI * HI' + M * R + M');
            
            z_(:,i) = [x_I(2,i) * g * x_I(1,i) + p0; x_I(2,i) * g * x_I(1,i) + p0 + x_I(2,i) * g * (d - h_r) + x_I(3,i) * q_in(k)^2];

            xpI(:,i) = x_I(:,i) + KI * (zc(:,k) - z_(:,i));
            PI = (eye(3) - KI * HI) * PI;
            
        elseif(i>1)
            PI = F * PI * F' + L * Q * L;
            x_I(:,i) = [xpI(1,i-1) + (1/A * (q_in(k) + q_f(k) - q_out(k)))/steps; xpI(2,i-1); xpI(3,i-1)];
            HI = [(x_I(2,i)) * g g * x_I(1,i) 0; (x_I(2,i)) * g g * x_I(1,i) + g * (d - h_r) q_in(k)^2];
            KI = PI * HI' * inv(HI * PI * HI' + M * R + M');         
            z_(:,i) = [x_I(2,i) * g * x_I(1,i) + p0; x_I(2,i) * g * x_I(1,i) + p0 + x_I(2,i) * g * (d - h_r) + x_I(3,i) * q_in(k)^2];

            xpI(:,i) = x_I(:,i) + KI * (zc(:,k) - z_(:,i));
            PI = (eye(3) - KI * HI) * PI;
          
        end
        

    end
    
    P = PI;
    x_(:,k) = x_I(:,steps);
    H = HI;
    K = KI;
    xp(:,k) = xpI(:,steps);
    P = PI;
    
    zp(:,k) = [xp(2,k) * g * xp(1,k) + p0; xp(2,k) * g * xp(1,k) + p0 + xp(2,k) * g * (d - h_r) + xp(3,k) * q_in(k)^2];

end

%Plots


figure
subplot(3,1,1)
plot(xc(1,1:3000)-xp(1,:))
axis([0 3000 -1 1])
xlabel('Time [s]')
ylabel('Height [m]')
grid on
title('Error in heightestimate')
subplot(3,1,2)
plot(xc(2,1:3000)-xp(2,:))
axis([0 3000 -1 1])
grid on
xlabel('Time [s]')
ylabel('Density [kg/m^3]')
title('Error in estimate of density')
subplot(3,1,3)
plot(xc(3,1:3000)-xp(3,:))
% axis([0 3000 -5e-6 5e-6])
grid on
xlabel('Time [s]')
ylabel('Density [kg/m^3]')
title('Error in estimate of friction factor')

figure
subplot(2,1,1)
plot(zp(2,:)/1e5-zc(2,:)/1e5)
% legend('bhp hat-bhp')
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('Error between estimated and real bhp')
axis([0 3000 -1e-3 1e-3])
grid on
subplot(2,1,2)
plot(zp(1,:)/1e5-zc(1,:)/1e5)
% legend('bhp hat-bhp')
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('Error between estimated and real pressure at the extraction pump')
axis([0 3000 -1e-3 1e-3])
grid on



figure
subplot(3,1,1)
plot(xp(1,:))
hold on
% plot(x(1,:),'r')
plot(xc(1,:),'--r')
axis([0 maxCounts 2020 2060])
legend('Estimated','Real')
% legend('Est','Noise','Perf')
xlabel('Time [s]')
ylabel('Height [m]')
title('Mudheight in riser')
grid on
subplot(3,1,2)
plot(xp(2,:))
hold on
% plot(x(2,:),'r')
plot(xc(2,:),'--r')
axis([0 maxCounts 1870 1890])
legend('Estimated','Real')
% legend('Est','Noise','Perf')
xlabel('Time [s]')
ylabel('Density [kg/m^3]')
title('Mudweight in annulus')
grid on
subplot(3,1,3)
plot(xp(3,:))
hold on
% plot(x(2,:),'r')
plot(xc(3,:),'--r')
% axis([0 maxCounts 1840 1860])
legend('Estimated','Real')
% legend('Est','Noise','Perf')
xlabel('Time [s]')
ylabel('Fa [kg/m^3]')
title('Friction coefficient')
grid on

figure
subplot(2,1,1)
plot(zp(2,:)/1e5)
hold on
% plot(zp(2,:)/1e5,'r')
plot(zc(2,:)/1e5,'--r')
legend('Estimated','Real')
axis([0 maxCounts 1670 1690])
% legend('Noise','Est','Perf')
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('Bottom hole pressure')
grid on
subplot(2,1,2)
plot(zp(1,:)/1e5)
hold on
% plot(zp(1,:)/1e5,'r')
plot(zc(1,:)/1e5,'--r')
axis([0 maxCounts 370 380])
legend('Estimated','Real')
xlabel('Time [s]')
ylabel('Pressure [bar]')
title('Pressure at extraction pump')
grid on

figure
plot(q_in*60000)
hold on
plot(q_f*60000,'k')
plot(q_out*60000,'r')
plot((q_in+q_f-q_out)*60000,'--g')
legend('q_{in}','q_f','q_{out}','q_{in}+q_f-q_{out}')
xlabel('Time [s]')
ylabel('Flow [l/minute]')
title('Flow')
grid on