% function simulateUsingR02(maxCounts,filename,plotDimension)
% Calculates drilling fluid flowrates and pressures
% (c) IRIS Petroleum
% 1. October 2008 GNY
% 12. Nov 2008 OEBR (oebr@iris.no)


reg = 1000; % How often the setpoint shall be altered
maxCounts = 18*reg; % The simulation length

plotThick = 1;

filename = 'test';

%Constants

g = 9.81;
d = 9220;
h_r = 2150;
rho_w = 0;
riserID = 0.473;
dsOD = 0.127;
A = pi*riserID^2 - pi*dsOD^2;
p0 = 1e5;


%Initial values for states

xp(:,1) = [2100;1750;1.13e8];
xpI(:,1) = xp(:,1);
xp2(:,1) = xp(:,1);
x_(:,1) = xp(:,1);
x_2(:,1) = xp(:,1);


P(:,:,1) = eye(3); %Error covariance matrix

% Jacobian matrices for the system

F = zeros(3);
L = eye(3);
M = eye(2);

%Process and measurement noise

sigmaw1 = 0.1;
sigmaw2 = 0.1;
sigmaw3 = 0.1;
sigmav1 = 200;
sigmav2 = 600;

%Process noise covariance matrix

Q = zeros(3);
Q(1,1) = 10000*sigmaw1^2;
Q(2,2) = 10000*sigmaw2^2;
Q(3,3) = 10000*sigmaw3^2;

%Measurement noise covraiance matrix

R = zeros(2);
R(1,1) = 10000*sigmav1^2;
R(2,2) = 10000*sigmav2^2;

%First timestep

u1Array(1) = 1000/60000;
u7Array(1) = 1000/60000;
u8Array(1) = 0;

z_(:,1) = [rho_w * g * (h_r - x_(1,1)) + x_(2,1) * g * x_(1,1) + p0; rho_w * g * (h_r - x_(1,1)) + x_(2,1) * g * x_(1,1) + p0 + x_(2,1) * g * (d - h_r) + x_(3,1) * u1Array(1)];
zp(:,1) = [rho_w * g * (h_r - xp(1,1)) + xp(2,1) * g * xp(1,1) + p0; rho_w * g * (h_r - xp(1,1)) + xp(2,1) * g * xp(1,1) + p0 + xp(2,1) * g * (d - h_r) + xp(3,1) * u1Array(1)];
zpK(:,1) = zp(:,1);
meas(:,1) = [zp(1,1); zp(2,1)];


  % Brukes midlertidig til Main Pump
  firstStartUp = 0;
  firstStartUpTime = 1*60;
  firstFlow    = 0/(60*1000);
  startFirstCounter     = 0;

  
  % Brukes midlertidig til Extraction Pump
  fourthStartUp = 0;
  fourthStartUpTime = 1*60;
  fourthFlow    = 000/(60*1000);
  startFourthCounter     = 0;
  
timeStep = 1; % 1 second per timeStep


caseNumber = 8; 
initWeMod(caseNumber);
    
% Defining initial u vector
% u1 - main pump rate, [m3/s] 
% u2 - choke opening, [m2]
% u3 - choke pump annulus pump rate, [m3/s] 
% u4 - fluid density, [kg/m3](alternative: [SG] (relative to water density 1)
% u5 - drillstring velocity  [m/s]
% u6 - choke pump fluid density  [kg/m3]
% u7 - annulus extraction pump flow rate  [m3/s]
% u8 - annulus injection pump flow rate  [m3/s]  
% u9 - annulus injection pump density  [kg/m3]
% u10 - Drillstring Rotation. [RPM]

u1 = 0;
u2 = 1; % m2, more than full opening...
u3 = 0; %Not available in this version...
u4 = 1759.7;
u5 = 0;
u6 = 1000;
u7 = 0;
u8 = 0;
u9 = 1760;
u10 = 0;




% DEFINE REFERENCE VALUES HERE:
% reference pressures, annulus velocities etc...
%
% end of reference definitions

% initialize controller
    
% end of initialize controller

% initialize observer

%end of initialize observer

%Define sensor selection
%numSensors = 12; % each having 9 values (pres + 7*flow + temp)
%numBranches = 5; % each having 8 values (pres + 7*flow)
%Define sensor selection
numSensors = 0; % each having 9 values (pres + 7*flow + temp)
numBranches = 4; % each having 8 values (pres + 7*flow)

% Extract sensor names
numSensorNames = calllib('WeModForMatlab','getSensorNameListSize');
% numSensorNames
for SensorNum = 1:numSensorNames
    SensorName = calllib('WeModForMatlab', 'getSensorName',SensorNum-1);
    disp(SensorName)
    save('namefile','SensorName');
end


%loop
disp(sprintf('Simulating to %d [s]',maxCounts));

u1Array = zeros(maxCounts,1);
u2Array = zeros(maxCounts,1);
u3Array = zeros(maxCounts,1);
u4Array = zeros(maxCounts,1);
u5Array = zeros(maxCounts,1);
u6Array = zeros(maxCounts,1);
u7Array = zeros(maxCounts,1);
u8Array = zeros(maxCounts,1);
u9Array = zeros(maxCounts,1);
u10Array = zeros(maxCounts,1);

U = zeros(10,maxCounts);
  spp = zeros(maxCounts,1);
  bhp= zeros(maxCounts,1);
  cdp= zeros(maxCounts,1);
 
  wm_bit= zeros(maxCounts,1);
  wm_pump= zeros(maxCounts,1);
  wm_exit= zeros(maxCounts,1);
  wm_inject= zeros(maxCounts,1);
  wb_bit= zeros(maxCounts,1);
  wb_pump= zeros(maxCounts,1);
  wb_exit= zeros(maxCounts,1);
  wb_inject= zeros(maxCounts,1);
  africFactors = zeros(maxCounts,1);
  dfricFactors = zeros(maxCounts,1);
  
for timeCount = 1:maxCounts
  % ADD CONTROLLER HERE
  % The u variables can either be control variables or measured
  % disturbances
  
  u1 = 1000/60000;
  
  %Tunable controller parameters
  
   Kp = 0.00000027; 
      Ti = 0.428;
      
      Kp2 = 0.0000027;
      Ti2 = 0.428;
  
%      eBHP(timeCount) = bhp(timeCount-1)/1e5-bhpRef(timeCount); %The control is done on the measured bottom hole pressure
      
     
      eBHP(timeCount) = zp(2,timeCount-1)/1e5-bhpRef(timeCount); %The control is done on the estimated bottom hole pressure



%The reference pressure is set
  if(timeCount>=3 && timeCount <=reg)
      bhpRef(timeCount) = 1685;
  end
  
  if(timeCount>reg && timeCount <=2*reg)
      bhpRef(timeCount) = 1680;   
  end
  
  if(timeCount>2*reg && timeCount <=3*reg)
      bhpRef(timeCount) = 1680;
  end
  
  if(timeCount>3*reg && timeCount <=4*reg)
      bhpRef(timeCount) = 1675;
  end
  
  if(timeCount>4*reg && timeCount <=5*reg)
      bhpRef(timeCount) = 1670;
  end
  
  if(timeCount>5*reg && timeCount <=6*reg)
      bhpRef(timeCount) = 1665;
  end
  
  if(timeCount>6*reg && timeCount <=7*reg)
      bhpRef(timeCount) = 1660;
  end
  
  if(timeCount>7*reg && timeCount <=8*reg)
      bhpRef(timeCount) = 1655;
  end
  
  if(timeCount>8*reg && timeCount <=9*reg)
      bhpRef(timeCount) = 1650;
  end
  
  if(timeCount>9*reg && timeCount <=10*reg)
      bhpRef(timeCount) = 1645;
  end
  
  if(timeCount>10*reg && timeCount <=11*reg)
      bhpRef(timeCount) = 1640;
  end
  
  if(timeCount>11*reg && timeCount <=12*reg)
      bhpRef(timeCount) = 1645;
  end
  
  if(timeCount>12*reg && timeCount <=13*reg)
      bhpRef(timeCount) = 1650;
  end
  
  if(timeCount>13*reg && timeCount <=14*reg)
      bhpRef(timeCount) = 1655;
  end
  if(timeCount>14*reg && timeCount <=15*reg)
      bhpRef(timeCount) = 1660;
  end
  
  if(timeCount>15*reg && timeCount <=16*reg)
      bhpRef(timeCount) = 1665;
  end
  
  if(timeCount>16*reg && timeCount <=17*reg)
      bhpRef(timeCount) = 1670;
  end
  
  if(timeCount>17*reg && timeCount <=18*reg)
      bhpRef(timeCount) = 1675;
  end
  
  if(timeCount>18*reg && timeCount <=19*reg)
      bhpRef(timeCount) = 1680;
  end
  
  if(timeCount>19*reg && timeCount <=20*reg)
      bhpRef(timeCount) = 1685;
  end
  
  %The PI regulator
  
      u7 = u7Array(timeCount-1) + Kp * (1 + 1/Ti) * eBHP(timeCount) - Kp * (1 - 1/Ti) * eBHP(timeCount-1);
      u8 = u8Array(timeCount-1) - Kp2 * (1 + 1/Ti2) * eBHP(timeCount) + Kp2 * (1 - 1/Ti2) * eBHP(timeCount-1);
  

  
      %The constraints for the pumps
  if(u7<0)
      u7 = 0;
  end
  if(u7>2000/60000)
     u7=2000/60000;
  end
  
  if(u8<0)
     u8 = 0;
  end
  if(u8>2000/60000)
     u8=2000/60000;
  end
  
    
  % Store controlled variables
  u1Array(timeCount) = u1;
  u2Array(timeCount) = u2;
  u3Array(timeCount) = u3;
  u4Array(timeCount) = u4;
  u5Array(timeCount) = u5;
  u6Array(timeCount) = u6;
  u7Array(timeCount) = u7;
  u8Array(timeCount) = u8;
  u9Array(timeCount) = u9;
  u10Array(timeCount) = u10;
 
  
  u = [u1,u2,u3,u4, u5, u6, u7, u8, u9, 10];
  U(:,timeCount) = u; 
    

[depthProfileAnnulus,...
          verticalDepthProfileAnnulus,...
          pressureProfileAnnulus,...
          velocityProfileAnnulus,...
          densityProfileAnnulus,...
          temperatureProfileAnnulus,...
          massProfileAnnulus,...
          volumeProfileAnnulus,...
          depthProfileDrillString,...
          pressureProfileDrillString,...
          velocityProfileDrillString,...
          maxVals] = stepWeMod(u);
  
%   depthProfileMatrixAnnulus(:,timeCount)  = depthProfileAnnulus;    
%   verticalDepthProfileMatrixAnnulus(:,timeCount)  =  verticalDepthProfileAnnulus;
%   pressureProfileMatrixAnnulus(:,timeCount)  = pressureProfileAnnulus;
%   velocityProfileMatrixAnnulus(:,timeCount)  = velocityProfileAnnulus;
%   densityProfileMatrixAnnulus(:,timeCount)  = densityProfileAnnulus;
%   temperatureProfileMatrixAnnulus(:,timeCount)  = temperatureProfileAnnulus;
%   massProfileMatrixAnnulus(:,timeCount)  = massProfileAnnulus;
%   volumeProfileMatrixAnnulus(:,timeCount)  = volumeProfileAnnulus;
% 
%   depthProfileMatrixDrillString(:,timeCount)  = depthProfileDrillString;    
%   pressureProfileMatrixDrillString(:,timeCount)  = pressureProfileDrillString;
%   velocityProfileMatrixDrillString(:,timeCount)  = velocityProfileDrillString;
  
  y = zeros(maxVals,1);
  % Reading sensor values using get methods
   for yPos = 1:maxVals
    y(yPos)  = calllib('WeModForMatlab', 'getMeasurementUsingPos',yPos-1);% C array indexes start at 0
   end
  
 
  %Friction parameters
  africFactors(timeCount) = calllib('WeModForMatlab', 'getAnnulusFrictionFactorParameter');
  dfricFactors(timeCount)  = calllib('WeModForMatlab', 'getDrillStringFrictionFactorParameter');
   
   
  %Replace with point data structure
  spp(timeCount) = calllib('WeModForMatlab', 'getSPP');  %Stand pipe pressure
  bhp(timeCount) = calllib('WeModForMatlab', 'getPressureAfterBit');   %Bottom hole pressure in annulus
  cdp(timeCount) = calllib('WeModForMatlab', 'getChokePressure');   %Choke differential pressure
  
  BHP = bhp(timeCount)/1e5;
     
       
   rTop(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',50);
   pTop(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',40);
  

  riserBottom(timeCount) = calllib('WeModForMatlab', 'getMeasurementUsingPos',45);

%  mud
  wm_bit(timeCount) =  -y(32);%-y(36); %Mass flow rate at bit
  wm_pump(timeCount) = y(37);%y(46); % at pump
  wm_exit(timeCount) =  -y(27);%-y(31); %and at exit
  wm_inject(timeCount) =  -y(52);%-y(61); %and at injection pipe

%baryte
  wb_bit(timeCount) = -y(33);%-y(37); %Mass flow rate at bit
  wb_pump(timeCount) = y(38);%y(47); % at pump
  wb_exit(timeCount) = -y(28);%-y(32); %and at exit
  wb_inject(timeCount) = -y(53);%-y(62); %and at injection pipe


  
  
  %%%%%%%%%%%%%%%%%
  % Kalman filter %
  %%%%%%%%%%%%%%%%%
  
  
  
   freq = 30; %The update frequency for the bottom hole pressure
  steps = 1; %The number of times the kalman filter is updated each timestep
  
  if(timeCount>1)

     
      if(timeCount<=freq)    
       
          for k=1:steps
              
              if(k==1)
                
                  PK = F * P(:,:,timeCount-1) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-1) + (1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)))/steps; xp(2,timeCount-1); xp(3,timeCount-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
                   
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];

                  
              elseif(k>1)
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)))/steps; xpK(2,k-1); xpK(3,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount)];
                  
                  measK(:,k) = [riserBottom(timeCount);z_K(2,k)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];

              end
          end
                
                
            P(:,:,timeCount) = PK;
            x_(:,timeCount) = x_K(:,steps);

            H(:,:,timeCount) = HK;
            K(:,:,timeCount) = KK;

            z_(:,timeCount) = z_K(:,steps);
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xpK(:,steps);
            P(:,:,timeCount) = PK;

            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) * u1Array(timeCount)];

        
      else
        
        if(floor((timeCount-1)/freq) == (timeCount-1)/freq && timeCount>=freq)
            bhpUpdate(timeCount) = bhp(timeCount-freq);
            
            for k=1:steps
                
                if(k==1)
                
                  PK = F * P(:,:,timeCount-freq) * F' + L * Q * L';
                  x_K(:,k) = [xp(1,timeCount-freq) + (1/A * (u1Array(timeCount-freq) + u8Array(timeCount-freq) - u7Array(timeCount-freq)))/steps; xp(2,timeCount-freq); xp(3,timeCount-freq)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount-freq)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount-freq)];
                   
                  measK(:,k) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];
                  
                elseif(k>1)
                  PK = F * PK * F' + L * Q * L';
                  x_K(:,k) = [xpK(1,k-1) + (1/A * (u1Array(timeCount-freq) + u8Array(timeCount-freq) - u7Array(timeCount-freq)))/steps; xpK(2,k-1); xpK(3,k-1)];
                  HK = [(x_K(2,k)-rho_w) * g g * x_K(1,k) 0; (x_K(2,k)-rho_w) * g g * x_K(1,k) + g * (d - h_r) u1Array(timeCount)];
                  KK = PK * HK' * inv(HK * PK * HK' + M * R + M');

                  z_K(:,k) = [rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0; rho_w * g * (h_r - x_K(1,k)) + x_K(2,k) * g * x_K(1,k) + p0 + x_K(2,k) * g * (d - h_r) + x_K(3,k) * u1Array(timeCount-freq)];
                  
                  measK(:,k) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                  xpK(:,k) = x_K(:,k) + KK * (measK(:,k) - z_K(:,k));
                  PK = (eye(3) - KK * HK) * PK;
                  zpK(:,k) = [rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0; rho_w * g * (h_r - xpK(1,k)) + xpK(2,k) * g * xpK(1,k) + p0 + xpK(2,k) * g * (d - h_r) + xpK(3,k) * u1Array(timeCount)];
                end

                PI(:,:,timeCount-freq) = PK;
                x_I(:,timeCount-freq) = x_K(:,steps);
                HI(:,:,timeCount-freq) = HK;
                KI(:,:,timeCount-freq) = KK;

                z_I(:,timeCount-freq) = z_K(:,steps);

                measI(:,timeCount-freq) = [riserBottom(timeCount-freq);bhp(timeCount-freq)];
                xpI(:,timeCount-freq) = xpK(:,steps);
                PI(:,:,timeCount-freq) = PK;
            end


                for j=(timeCount-freq+1):(timeCount)
                PI(:,:,j) = F * PI(:,:,j-1) * F' + L * Q * L';
                x_I(:,j) = [xpI(1,j-1) + 1/A * (u1Array(j) + u8Array(j) - u7Array(j)); xpI(2,j-1); xpI(3,j-1)];

                HI(:,:,j) = [(x_I(2,j)-rho_w) * g g * x_I(1,j) 0; (x_I(2,j)-rho_w) * g g * x_I(1,j) + g * (d - h_r) u1Array(j)];
                KI(:,:,j) = PI(:,:,j) * HI(:,:,j)' * inv(HI(:,:,j) * PI(:,:,j) * HI(:,:,j)' + M * R + M');

                z_I(:,j) = [rho_w * g * (h_r - x_I(1,j)) + x_I(2,j) * g * x_I(1,j) + p0; rho_w * g * (h_r - x_I(1,j)) + x_I(2,j) * g * x_I(1,j) + p0 + x_I(2,j) * g * (d - h_r) + x_I(3,j) * u1Array(j)];

                measI(:,j) = [riserBottom(j);z_I(2,j)];

                xpI(:,j) = x_I(:,j) + KI(:,:,j) * (measI(:,j) - z_I(:,j));
                zpI(:,j) = [rho_w * g * (h_r - xpI(1,j)) + xpI(2,j) * g * xpI(1,j) + p0; rho_w * g * (h_r - xpI(1,j)) + xpI(2,j) * g * xpI(1,j) + p0 + xpI(2,j) * g * (d - h_r) + xpI(3,j) * u1Array(j)];

                PI(:,:,j) = (eye(3) - KI(:,:,j) * HI(:,:,j)) * PI(:,:,j);
            end

            P(:,:,timeCount) = PI(:,:,timeCount);
            x_(:,timeCount) = x_I(:,timeCount);

            H(:,:,timeCount) = HI(:,:,timeCount);
            K(:,:,timeCount) = KI(:,:,timeCount);

            z_(:,timeCount) = z_I(:,timeCount);
            
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = xpI(:,timeCount);
            
            zp(:,timeCount) = zpI(:,timeCount);
            


        else
            
            P(:,:,timeCount) = F * P(:,:,timeCount-1) * F' + L * Q * L';
            x_(:,timeCount) = [xp(1,timeCount-1) + 1/A * (u1Array(timeCount) + u8Array(timeCount) - u7Array(timeCount)); xp(2,timeCount-1); xp(3,timeCount-1)];

            H(:,:,timeCount) = [(x_(2,timeCount)-rho_w) * g g * x_(1,timeCount) 0; (x_(2,timeCount)-rho_w) * g g * x_(1,timeCount) + g * (d - h_r) u1Array(timeCount)];
            K(:,:,timeCount) = P(:,:,timeCount) * H(:,:,timeCount)' * inv(H(:,:,timeCount) * P(:,:,timeCount) * H(:,:,timeCount)' + M * R + M');

            z_(:,timeCount) = [rho_w * g * (h_r - x_(1,timeCount)) + x_(2,timeCount) * g * x_(1,timeCount) + p0; rho_w * g * (h_r - x_(1,timeCount)) + x_(2,timeCount) * g * x_(1,timeCount) + p0 + x_(2,timeCount) * g * (d - h_r) + x_(3,timeCount) * u1Array(timeCount)];
            meas(:,timeCount) = [riserBottom(timeCount);z_(2,timeCount)];

            xp(:,timeCount) = x_(:,timeCount) + K(:,:,timeCount) * (meas(:,timeCount) - z_(:,timeCount));
            P(:,:,timeCount) = (eye(3) - K(:,:,timeCount) * H(:,:,timeCount)) * P(:,:,timeCount);
            
            zp(:,timeCount) = [rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0; rho_w * g * (h_r - xp(1,timeCount)) + xp(2,timeCount) * g * xp(1,timeCount) + p0 + xp(2,timeCount) * g * (d - h_r) + xp(3,timeCount) * u1Array(timeCount)];

        end
        
      end
      
      
  end
  
  
  %
  % End of observer code

  % ADD Evaluation of reference here
  %
  % End of evaluating reference
  
  %Show progress
  if rem(timeCount,floor(100/timeStep)) == 0
    disp(sprintf('Time: %d [s] and BHP: %10.5f [bar]',floor(timeCount*timeStep),bhp(timeCount)/1e5));
  end
end

save(filename,'U','spp','bhp','cdp');

endWeMod(caseNumber);


%%
%Various plots

figure
plot(bhp/1e5)
hold on
plot(bhpRef,'r')
axis([0 maxCounts 1630 1690])

figure
plot(u1Array*60000)
hold on
grid on
plot(u8Array*60000,'g')
plot(u7Array*60000,'r')
axis([0 maxCounts 0 2000])
xlabel('Time [s]')
ylabel('Flow [l/min]')
legend('q_{in}','q_{fill}','q_{out}')
title('The flow in the system')

% figure
% subplot(3,1,1)
% plot(bhp/1e5)
% hold on
% plot(bhpRef,'--r')
% grid on
% axis([0 maxCounts 1635 1690])
% legend('BHP','Reference')
% xlabel('Time [s]')
% ylabel('Pressure [bar]')
% title('The estimated bottom hole pressure plotted together with the reference pressure')
% subplot(3,1,2)
% plot(bhp/1e5-bhpRef')
% grid on
% axis([0 maxCounts -6 6])
% xlabel('Time [s]')
% ylabel('Pressure [bar]')
% title('The difference between the setpoint and the bottom hole pressure')
% subplot(3,1,3)
% plot(u1Array*60000)
% hold on
% grid on
% plot(u8Array*60000,'g')
% plot(u7Array*60000,'r')
% axis([0 maxCounts 0 2000])
% xlabel('Time [s]')
% ylabel('Flow [l/min]')
% legend('q_{in}','q_{fill}','q_{out}')
% title('The flow in the system')
% 
% figure
% subplot(2,1,1)
% plot(zp(2,:)/1e5)
% hold on
% plot(bhp/1e5,'--r')
% % plot(bhpUpdate/1e5,'.k')
% xlabel('Time [s]')
% ylabel('Pressure [bar]')
% legend('Estimated','Measured')
% title('Bottom hole pressure')
% grid on
% axis([0 maxCounts 1635 1690])
% subplot(2,1,2)
% plot(zp(1,:)/1e5)
% hold on
% plot(riserBottom/1e5,'--r')
% xlabel('Time [s]')
% ylabel('Pressure [bar]')
% legend('Estimated','Measured')
% title('Pressure at extraction pump')
% grid on
% axis([0 maxCounts 335 385])
% % 
% figure
% subplot(2,1,1)
% plot(zp(2,:)/1e5-bhp'/1e5)
% xlabel('Time [s]')
% ylabel('Pressure [bar]')
% title('Error between estimated and measured bottom hole pressure')
% grid on
% axis([0 maxCounts -3 3])
% subplot(2,1,2)
% plot(zp(1,:)/1e5-riserBottom/1e5)
% xlabel('Time [s]')
% ylabel('Pressure [bar]')
% title('Error between estimated and measured pressure at the extraction pump')
% grid on
% axis([0 maxCounts -1e-2 1e-2])

% figure
% subplot(2,1,1)
% plot(xp(1,:))
% xlabel('Time [s]')
% ylabel('Height [m]')
% title('Height of mud in the riser')
% % axis([0 maxCounts 1850 2020])
% grid on
% subplot(2,1,2)
% plot(xp(2,:))
% xlabel('Time [s]')
% ylabel('Weight [kg/m^3]')
% title('Density of the mud')
% % axis([0 maxCounts 1870 1900])
% grid on
% 
% 
% figure
% plot(bhpUpdate/1e5,'.b')
% hold on
% plot(bhp/1e5,'r')
% plot(zp(2,:)/1e5,'g')
% % plot(z(2,:)/1e5,'--k')
% % legend('Measurement','Calculated')
% legend('Update of measurement','Measurement','Calculated')
% xlabel('Time [s]')
% ylabel('Pressure [bar]')
% title('The bottom hole pressure in the well')
% grid on
% % axis([0 3000 1685 1715])

